<?php

namespace BayCMS\Base;

class BayCMSAuthenticator
{
    protected BayCMSContext $context;

    private $AUTH_OK = false;
    private $IP_AUTH_OK = false;

    public function __construct(BayCMSContext $context)
    {
        $this->context = $context;
    }

    public function authenticated()
    {
        return $this->AUTH_OK;
    }

    public function authPage($login = '', $logout = false)
    {
        $ls_link = $this->context->get('ls_link');
        if ($this->context->get('row1')['loginlogout'] == "t")
            header("Location: /$ls_link/" . $this->context->get('lang') . "/top/gru/login.php?login=$login&url=" .
                rawurlencode("$_SERVER[PHP_SELF]?" . str_replace("aktion=", "aktion_disabled=", $_SERVER['QUERY_STRING'])) .
                ($logout ? "&aktion=logout" : ""));
        else {
            Header("HTTP/1.0 401 Unauthorized");
            Header("WWW-Authenticate: Basic realm=\"$ls_link\"");
        }
        echo "<head>
  <meta http-equiv=\"refresh\" content=\"0; URL=http://$_SERVER[HTTP_HOST]/$ls_link/\">
  </head>
  Access denied\n";
        exit();

    }

    public function logout()
    {
        pg_query_params(
            $this->context->getRwDbConn(),
            'delete from session where id_session=$1',
            [session_id()]
        );
        $this->AUTH_OK = false;
        unset($_SESSION['no_ipauth']);

    }

    public function authenticateBySession($SID = null)
    {
        if ($SID === null)
            $SID = session_id();
        $row1 = $this->context->get('row1');
        $res = pg_query_params(
            $this->context->getDbConn(),
            "select b.*,s.*,s.timeout<(now()+interval '30 minutes') as update_session
  		from session s, benutzer b where s.id_session=\$1 and
  		s.id_benutzer=b.id and s.id_lehr=\$2 and (s.power>=\$3
  		or hat_zugang(\$2,b.id,\$4))",
            [
                $SID,
                $row1['id'],
                $this->context->get('min_power'),
                $this->context->get('kat_id')
            ]
        );
        if (pg_num_rows($res)) {
            $r = pg_fetch_array($res, 0);
            if ($r['update_session'] == 't') {
                pg_query_params(
                    $this->context->getRwDbConn(),
                    'update session set timeout=now()+interval \'1 hour\' where id_session=$1 and id_lehr=$2',
                    [$SID, $row1['id']]
                );
            }
            $this->AUTH_OK = true;
            $_SESSION['no_ipauth'] = 1;
            $row1['id_benutzer'] = $r['id'];
            $row1['power'] = $r['power'];
            $row1['kommentar'] = $r['kommentar'];
            $row1['user_email'] = $r['email'];
            $row1['login'] = $r['login'];
            $this->context->set('row1', $row1);
            $this->context->set('AUTH_OK', true);
            return;
        }

        //Try to catch up session from an other org
        //Some variables used in old auth fallback scripts...
        $conn3 = $this->context->getRwDbConn();
        $conn1 = $this->context->getDbConn();
        $SITE_ENCODING = 'UTF-8';
        $DOC_PFAD = $this->context->get('BayCMSRoot');
        $DB_OWNER = $this->context->get('DB_OWNER');


        $res = pg_query_params(
            $this->context->getDbConn(),
            "select b.*
		from session s, benutzer b where s.id_session=\$1 and
		s.id_benutzer=b.id",
            array($SID)
        );
        if (!pg_num_rows($res))
            return;

        $r = pg_fetch_array($res, 0);
        $res = pg_query($this->context->getDbConn(), "select b.*,
            il.last_login is null or il.last_login<(now()-interval '0.5 day') as update_lastlogin,
			il.id_benutzer as id_eff 
            from
			(select *,
            get_max_login_power(id,$row1[id]) as power,hat_zugang($row1[id],id," . $this->context->get('kat_id') . ")
			from benutzer where id=$r[id]) b,in_ls il
			where il.power=b.power and 
            (il.id_lehr=$row1[id] and in_gruppe(b.id,il.id_benutzer)) or il.power>=10000");
        if (pg_num_rows($res))
            $r = pg_fetch_array($res, 0);
        else
            $r = array('hat_zugang' => 'f', 'power' => 0);

        if ($this->context->get('min_power') > $r['power'] && $r['hat_zugang'] != "t") {
            $auth_fallback_ok = 0;
            //Fallback Authentisierung fuer spezielle Zugangsregeln:
            //bei Erfolg werden $auth_fallback_ok und $r gesetzt:
            $res = pg_query(
                $this->context->getDbConn(),
                "select f.name from file f,auth_fallback a where a.id_lehr=$row1[id] 
                and a.id_file=f.id and typ='zugang'"
            );
            $this->context->setSystemUser();
            for ($i = 0; $i < pg_num_rows($res); $i++) {
                [$auth_fallback] = pg_fetch_row($res, $i);
                include($this->context->get('BayCMSRoot') . "/$auth_fallback");
            }
            $this->context->setSystemUser(false);

            if (!$auth_fallback_ok)
                $r = array();
        }

        //Zugang wird gewährt
        if ($r['power'] && ($r['power'] >= $this->context->get('min_power') || $r['hat_zugang'] == "t")) {

            pg_query($this->context->getRwDbConn(), "begin");
            pg_query_params($this->context->getRwDbConn(), 'delete from session where timeout<now()
					or (id_session=$1 and id_lehr=$2)', array($SID, $row1['id']));

            if (!$no_session)
                pg_query_params(
                    $$this->context->getRwDbConn(),
                    'insert into session(id_session,id_benutzer,id_lehr,power,login,pw,timeout)
						values ($1,$2,$3,$4,$5,$6,now()+interval \'1 hour\')',
                    array($SID, $r['id'], $row1['id'], $r['power'], $r['login'], $r['pw'])
                );
            pg_query($this->context->getRwDbConn(), "commit");
            $_SESSION['no_ipauth'] = 1;
            if ($r['update_lastlogin'] != "f") {
                pg_query($this->context->getRwDbConn(), "update in_ls set last_login=now() where
						(id_benutzer=" . $r['id'] . " or id_benutzer=" . $r['id_eff'] . ") and id_lehr=" . $row1['id']);
            }
            $this->AUTH_OK = true;
            $row1['id_benutzer'] = $r['id'];
            $row1['power'] = $r['power'];
            $row1['kommentar'] = $r['kommentar'];
            $row1['user_email'] = $r['email'];
            $row1['login'] = $r['login'];
            $this->context->set('row1', $row1);
            $this->context->set('AUTH_OK', true);
            return;
        }

    }

    public function authenticateByIP()
    {
        $row1 = $this->context->get('row1');
        if (($row1['id_auth'] ?? 'f') != 't')
            return;
        if ($_SESSION['no_ipauth'] ?? false)
            return;
        if ($this->context->get('min_power') == 0)
            return;
        if (isset($_SERVER['PHP_AUTH_USER']) || isset($_POST['PHP_AUTH_USER']))
            return;

        if ($this->context->get('kat_id'))
            $H_query = "hat_zugang($row1[id],b.id," . $this->context->get('kat_id') . ")";
        else
            $H_query = "false as hat_zugang";
        $res = pg_query(
            $this->context->getDbConn(),
            "select b.*,$H_query,get_max_login_power(b.id,$row1[id]) as power from benutzer b,
      		benutzer_ip bi where b.id=bi.id_benutzer and '$_SERVER[REMOTE_ADDR]'<<=bi.ip
      		and (get_max_login_power(b.id,$row1[id])>0 or b.id in 
            (select id_benutzer from hat_zugang where id_lehr=$row1[id])) 
            order by power desc"
        );
        if (pg_num_rows($res)) {
            $r = pg_fetch_array($res, 0);
            $row1['id_benutzer'] = $r['id'];
            $row1['ip_power'] = $r['power'];
            $row1['power'] = 0;
            $row1['kommentar'] = $r['kommentar'];
            $row1['user_email'] = $r['email'];
            $row1['hat_zugang'] = $r['hat_zugang'];
            $this->context->set('row1', $row1);
            $this->IP_AUTH_OK = true;
            $this->context->set('IP_AUTH_OK', true);
        }


    }

    public function authenticateByUserPW($PHP_AUTH_USER, $PHP_AUTH_PW)
    {
        $ok = 0;
        //Some variables used in old auth fallback scripts...
        $conn3 = $this->context->getRwDbConn();
        $conn1 = $this->context->getDbConn();
        $SITE_ENCODING = 'UTF-8';
        $DOC_PFAD = $this->context->get('BayCMSRoot');
        $DB_OWNER = $this->context->get('DB_OWNER');
        $min_power = $this->context->get('min_power') ?? 0;



        $this->AUTH_OK = false;
        $PHP_AUTH_USER = strtolower(trim($PHP_AUTH_USER));
        $row1 = $this->context->get('row1');

        $res = pg_query_params($this->context->getDbConn(), 'select b.*,
           case when length(b.pw)>0 then pg_crypt($1,substr(b.pw,1,2)) end as pg_crypt,pw.bind_url,
                md5($1||b.salt) 
				from benutzer b
				left outer join pw_source pw on pw.id=b.id_pw_source 
				where b.login=$2 and not b.gruppe', array($PHP_AUTH_PW, $PHP_AUTH_USER));
        if ($ok = pg_num_rows($res))
            $r = pg_fetch_array($res, 0);//Benutzer bekannt
        if ($ok && !$r['pw_md5'] && $r['pw'] && $r['pw'] == $r['pg_crypt']) {
            pg_query_params(
                $this->context->getRwDbConn(),
                'update benutzer set pw_md5=$1,pw=\'\' where id=$2',
                [$r['md5'], $r['id']]
            );
            $r['pw_md5'] = $r['md5'];
        }
        if ($ok && $r['pw_md5'] != $r['md5']) {//PW falsch
            $ok = 0;
            //setzt $ok auf 1 falls erfolgreich
            if ($r['bind_url']) {
                if (preg_match('|^http://([^/]+)(/.*$)|', $r['bind_url'], $match)) {

                    if ($fp = fsockopen($match[1], 80)) {
                        fputs($fp, "GET $match[2]  HTTP/1.1\r\n");
                        fputs($fp, "Host: $match[1]\r\n");
                        fputs($fp, "Authorization: Basic " . base64_encode("$PHP_AUTH_USER:$PHP_AUTH_PW") . "\r\n");
                        fputs($fp, "Connection: close\r\n\r\n");
                        if (strstr(fgets($fp, 128), "AUTHORIZATION OK"))
                            $ok = 1;
                    }
                    fclose($fp);
                }

                // ldap://someserver.org/ou=People,dc=someserver,dc=org
                if ($PHP_AUTH_PW && preg_match('|^ldap://([^/]+)/(.*)$|', $r['bind_url'], $match)) {
                    if ($ds = ldap_connect($match[1], 389)) {
                        ldap_set_option($ds, LDAP_OPT_PROTOCOL_VERSION, 3);
                        if (ldap_bind($ds, "uid=$PHP_AUTH_USER,$match[2]", $PHP_AUTH_PW))
                            $ok = 1;
                    }

                }
                // ldaps://someserver.org/ou=People,dc=someserver,dc=org
                if ($PHP_AUTH_PW && preg_match('|(^ldaps://[^/]+)/(.*)$|', $r['bind_url'], $match)) {
                    if ($ds = ldap_connect($match[1])) {
                        ldap_set_option($ds, LDAP_OPT_PROTOCOL_VERSION, 3);
                        if (ldap_bind($ds, "cn=$PHP_AUTH_USER,$match[2]", $PHP_AUTH_PW))
                            $ok = 1;
                    }

                }

            }
            if ($ok && $r['sync_with_source'] == "t") {
                pg_query_params(
                    $this->context->getRwDbConn(),
                    'update benutzer set pw_md5=$1 where id=$2',
                    array($r['md5'], $r['id'])
                );
            }

        }
        //Login/PW-Kombination nicht korrekt
        if (!$ok) {
            $auth_fallback_ok = 0;
            $res = pg_query(
                $this->context->getDbConn(),
                "select f.name from file f,auth_fallback a 
					where a.id_lehr=$row1[id]
					and a.id_file=f.id and typ='benutzer'"
            );
            $this->context->setSystemUser();
            for ($i = 0; $i < pg_num_rows($res); $i++) {
                list($auth_fallback) = pg_fetch_row($res, $i);
                include($this->context->get('BayCMSRoot') . "/$auth_fallback");
            }
            $this->context->setSystemUser(false);
            if ($auth_fallback_ok)
                $ok = 1;
        }

        //Falls Login/PW korrekt -> Pruefe Zugang
        if ($ok) {
            $res = pg_query(
                $this->context->getDbConn(),
                "select b.*,
            il.last_login is null or il.last_login<(now()-interval '0.5 day') as update_lastlogin,
			il.id_benutzer as id_eff from
			(select *,get_max_login_power(id,$row1[id]) as power,hat_zugang($row1[id],id," . $this->context->get('kat_id') . ")
			from benutzer where id=$r[id]) b,in_ls il
			where il.power=b.power and (il.id_lehr=$row1[id] and in_gruppe(b.id,il.id_benutzer)) or il.power>=10000"
            );
            if (pg_num_rows($res))
                $r = pg_fetch_array($res, 0);
            else
                $r = array();

            if ($min_power > $r['power'] && $r['hat_zugang'] != "t") {
                $auth_fallback_ok = 0;
                //Fallback Authentisierung fuer spezielle Zugangsregeln:
                //bei Erfolg werden $auth_fallback_ok und $r gesetzt:
                $res = pg_query(
                    $this->context->getDbConn(),
                    "select f.name from file f,auth_fallback a where a.id_lehr=$row1[id] 
                and a.id_file=f.id and typ='zugang'"
                );
                $this->context->setSystemUser();
                for ($i = 0; $i < pg_num_rows($res); $i++) {
                    list($auth_fallback) = pg_fetch_row($res, $i);
                    include($this->context->get('BayCMSRoot') . "/$auth_fallback");
                }
                $this->context->setSystemUser(false);
                if (!$auth_fallback_ok)
                    $r = array();
            }

            if ($r['power'] && ($r['power'] >= $min_power || $r['hat_zugang'] == "t")) {
                //Delete old tmp files...
                $files = glob($this->context->get('BayCMSRoot') . '/tmp/*');
                $threshold = strtotime('-2 hours');
                foreach ($files as $file) {
                    if (is_file($file)) {
                        if ($threshold >= filemtime($file)) {
                            unlink($file);
                        }
                    }
                }

                pg_query($this->context->getRwDbConn(), "begin");
                pg_query_params(
                    $this->context->getRwDbConn(),
                    'delete from session where timeout<now() or (id_session=$1 and id_lehr=$2)',
                    [session_id(), $row1['id']]
                );
                pg_query_params(
                    $this->context->getRwDbConn(),
                    'insert into session(id_session,id_benutzer,id_lehr,power,login,pw,timeout) 
						values ($1,$2,$3,$4,$5,$6,now()+interval \'1 hour\')',
                    [session_id(), $r['id'], $row1['id'], $r['power'], $r['login'], $r['pw']]
                );
                pg_query($this->context->getRwDbConn(), "commit");
                $_SESSION['no_ipauth'] = 1;
                if ($r['update_lastlogin'] != "f") {
                    pg_query(
                        $this->context->getRwDbConn(),
                        "update in_ls set last_login=now() where 
							(id_benutzer=" . $r['id'] . " or id_benutzer=" . $r['id_eff'] . ") 
                            and id_lehr=" . $row1['id']
                    );
                }
                $this->AUTH_OK = true;
            }
        } else
            sleep(3);


        if ($this->AUTH_OK) {
            $row1['id_benutzer'] = $r['id'];
            $row1['power'] = $r['power'];
            $row1['kommentar'] = $r['kommentar'];
            $row1['user_email'] = $r['email'];
            $this->context->set('row1', $row1);
            $this->context->set('AUTH_OK', true);
        }

    }


    public function authenticate()
    {
        if (isset($_GET['aktion']) && $_GET['aktion'] == "logout") {
            $this->logout();
        }
        $this->authenticateBySession();
        $this->authenticateByIP();
        if (isset($_SERVER['PHP_AUTH_USER']))
            $this->authenticateByUserPW($_SERVER['PHP_AUTH_USER'], $_SERVER['PHP_AUTH_PW']);
        if (isset($_POST['PHP_AUTH_USER']))
            $this->authenticateByUserPW($_POST['PHP_AUTH_USER'], $_POST['PHP_AUTH_PW']);

        if ($this->context->get('min_power') > 0 && !$this->AUTH_OK)
            $this->authPage();

    }
}