<?php
namespace BayCMS\Base;

class BayCMSContext
{
    private array $context = [];

    private mixed $conn_ro = null;
    private mixed $conn_rw = null;

    private array $tr = [];

    public function __construct(array $context)
    {
        $this->context = $context;
        require $this->context['BayCMSRoot'] . "/inc/localconfig.inc";
        if (isset($DOC_PFAD))
            $this->context['BayCMSRoot'] = $DOC_PFAD;
        $this->context['DB_EXTERN'] = $DB_EXTERN;
        $this->context['DB_OWNER'] = $DB_OWNER;
        $this->context['DB_NAME'] = $DB_NAME;
        $this->context['HTTP_PATH'] = $HTTP_PFAD;
        $this->context['SITE_ENCODING'] = 'UTF-8';
        $this->context['IP_AUTH_OK'] = false;
        $this->context['AUTH_OK'] = false;


        $this->conn_ro = pg_connect($DB_EXTERN);
        pg_set_client_encoding($this->conn_ro, 'UTF-8');
        $this->context['conn1'] = $this->conn_ro;

        session_start();
        $this->context['SID'] = session_id();

        if (!isset($this->context['row1'])) {
            if (!isset($_SERVER['HTTP_HOST'])) {
                throw new \Exception('No way to create context');
            }
            // Find organisation ID
            if ($_SERVER['PHP_SELF'] != "/index.php") {
                $res = pg_query_params(
                    $this->conn_ro,
                    "select * from lehrstuhl where \$1 like link||'%' order by length(link) desc limit 1",
                    [substr($_SERVER['PHP_SELF'], 1)]
                );
            } else {
                [$dummy, $ls_link] = explode(".", $_SERVER['HTTP_HOST'], 3);
                $ls_link = strtolower($ls_link);
                $res = pg_query_params(
                    $this->conn_ro,
                    "select * from lehrstuhl where link=\$1 or (httphost_selector>'' and \$2 ilike httphost_selector)",
                    [$ls_link, $_SERVER['HTTP_HOST']]
                );
            }
            if (!pg_num_rows($res)) {
                $res = pg_query_params(
                    $this->conn_ro,
                    "select * from lehrstuhl where link=\$1",
                    [$LS_DEFAULT]
                );
            }

            if (pg_num_rows($res)) {
                $this->context['row1'] = pg_fetch_array($res, 0);
                $this->context['ls_link'] = $this->context['row1']['link'];
            } else {
                throw new \Exception('Unknown organisation');
            }
            $this->context['row1']['power'] = 0;


            // Parsen des Pfads: /$ls_link/$lang/$kategorie/$file
            $subpath = preg_replace(
                '&^/' . $this->context['ls_link'] . '/(.*?\.php).*$&',
                '\1',
                $_SERVER['PHP_SELF']
            );
            if ($subpath != 'index.php') {
                [$this->context['lang'], $this->context['kategorie'], $this->context['file']] = explode(
                    "/",
                    $subpath,
                    3
                );
                [$this->context['modul'], $this->context['php_file']] = explode("/", $this->context['file'], 2);
            }
            $this->setLang();
        }
    }

    public static function autoCreate($dir)
    {
        $try = ['../../', '', '../', '../../../'];
        foreach ($try as $d) {
            if (is_readable($dir . '/' . $d . 'inc/localconfig.inc')) {
                $context = new BayCMSContext(['BayCMSRoot' => $dir . '/' . $d]);
                $context->init();
                return $context;
            }
        }
        throw new \BayCMS\Exception\notFound('Did not find localconfig.inc');
    }

    public function t(string $en, string $de = null, bool $save = false): string
    {
        if ($save && !is_null($de))
            $this->tr[$en] = $de;
        if ($this->get('lang') == 'en')
            return $en;
        if (!is_null($de))
            return $de;
        return $this->tr[$en] ?? $en;
    }

    public function _404($msg = '')
    {
        $_SESSION['last_error_msg'] = $msg;
        header("Location: /" . $this->getLsLinkLang() . "/top/gru/404.php");
        exit();
    }

    public function _401($msg = '')
    {
        $_SESSION['last_error_msg'] = $msg;
        header("Location: /" . $this->getLsLinkLang() . "/top/gru/401.php");
        exit();
    }
    public function checkMinPower()
    {
        if (!isset($this->context['min_power']))
            $this->context['min_power'] = 0;

        $res = pg_query(
            $this->conn_ro,
            "select case when a.min_power>k.min_power then a.min_power else k.min_power end,
          non_empty(non_empty(a." . $this->context['lang'] . ",a." . $this->context['lang2'] . "),
          non_empty(k." . $this->context['lang'] . ",k." . $this->context['lang2'] . ")) as kat,
          k.id 
          from kategorie k left outer
          join (select * from kat_aliases where id_lehr=" . $this->context['row1']['id'] . ") a on k.id=a.id_kat 
          where k.link='" . $this->context['kategorie'] . "'"
        );
        if (pg_num_rows($res))
            [$this->context['kat_min_power'], $this->context['header_kat'], $this->context['kat_id']] =
                pg_fetch_row($res, 0);
        else {
            $this->context['kat_min_power'] = 0;
            $this->context['header_kat'] = '';
            $this->context['kat_id'] = 1000; //Kategorie TOP
        }
        if ($this->context['kat_min_power'] > $this->context['min_power'])
            $this->context['min_power'] = $this->context['kat_min_power'];
        if ($this->context['header_kat'] && $this->context['kategorie'] != "top")
            $this->context['header_kat'] = ": " . $this->context['header_kat'];
        else
            $this->context['header_kat'] = '';
        $this->context['title'] .= $this->context['header_kat'];
    }

    public function getPrePostHtml()
    {
        $this->context['page_prefix'] = '';
        $this->context['page_postfix'] = '';
        $res = pg_query($this->conn_ro, "select 
        non_empty(" . $this->getLangLang2("prefix_") . ") as prefix,
        non_empty(" . $this->getLangLang2('postfix_') . ") as postfix
        from pre_post_html where
        id_lehr=" . $this->context['row1']['id'] . " and
        '" . pg_escape_string(
                $this->conn_ro,
                $this->context['kategorie'] . "/" . $this->context['modul'] . "/" . $this->context['php_file'] . ($_SERVER['QUERY_STRING'] ? "?$_SERVER[QUERY_STRING]" : "")
            ) . "' ilike replace(path,'_','\\_')||'%'
                order by path limit 1");

        if (pg_num_rows($res)) {
            [$this->context['page_prefix'], $this->context['page_postfix']] = pg_fetch_row($res, 0);
        }

    }

    public function setLang($lang = null)
    {

        if ($lang !== null) {
            $this->context['lang'] = $lang;
        } elseif (isset($_GET['lang'])) {
            $this->context['lang'] = $_GET['lang'];
        }
        if (!isset($this->context['lang']))
            $this->context['lang'] = $this->context['row1']['lang'] ?? 'de';
        if (!in_array($this->context['lang'], ['de', 'en']))
            $this->context['lang'] = 'de';
        $this->context['lang2'] = $this->context['lang'] == 'de' ? 'en' : 'de';
        $this->context['title'] = $this->getRow1String('');

    }

    public function get($key = null, $key2 = null, $default = null): mixed
    {
        if ($key === null)
            return $this->context;
        if ($key2 !== null)
            return $this->context[$key][$key2] ?? $default;
        return $this->context[$key] ?? $default;
    }

    public function set($key, $value)
    {
        $this->context[$key] = $value;
    }

    public function getDbConn(): mixed
    {
        if ($this->conn_ro === false) {
            throw new \Exception('No database connection');
        }
        return $this->conn_ro;
    }
    public function getRwDbConn(): mixed
    {
        if ($this->conn_rw === null) {
            $this->conn_rw = pg_connect($this->context['DB_OWNER']);
            pg_set_client_encoding($this->conn_rw, 'UTF-8');
            $this->context['conn3'] = $this->conn_rw;
        }
        if ($this->conn_rw === false) {
            throw new \Exception('No database connection');
        }
        return $this->conn_rw;
    }



    public function registerGlobal()
    {
        trigger_error('Use of registerGlobal should be avoided for new scripts.');
        foreach ($this->context as $key => $value) {
            $GLOBALS[$key] = $value;
        }
    }

    public function registerMinimum()
    {
        $GLOBALS['row1'] = $this->context['row1'];
        $GLOBALS['lang'] = $this->context['lang'];
        $GLOBALS['lang2'] = $this->context['lang2'];
        $GLOBALS['ls_link'] = $this->context['ls_link'];
        $GLOBALS['conn1'] = $this->context['conn1'];
        $GLOBALS['SID'] = $this->context['SID'];
    }

    public function getIndexQueries(): array
    {
        $row1 = $this->context['row1'];
        $lang = $this->context['lang'];
        $lang2 = $this->context['lang2'];
        $ls_link = $this->context['ls_link'];
        if ($this->context['IP_AUTH_OK'] ?? false)
            $H_kat_query = "and
  (((
  (k.min_power<=$row1[ip_power] and (a.min_power is null or a.min_power<=$row1[ip_power]))
  or
  (k.min_power=0 and a.min_power>0 and a.min_power<=$row1[ip_power])))
  or ((k.min_power>$row1[ip_power] or a.min_power>$row1[ip_power])
  and hat_zugang($row1[id],$row1[id_benutzer],k.id))) ";
        elseif (($this->context['min_power'] ?? 0) && ($this->context['AUTH_OK'] ?? false))
            $H_kat_query = "and
  (((
  (k.min_power<=$row1[power] and k.min_power>0 and (a.min_power is null or a.min_power<=$row1[power]))
  or
  (k.min_power=0 and a.min_power>0 and a.min_power<=$row1[power])))
  or
  ((k.min_power>$row1[power] or a.min_power>$row1[power]) and hat_zugang($row1[id],$row1[id_benutzer],k.id))) ";
        else
            $H_kat_query = "and (k.min_power=0 and (a.min_power is null or a.min_power=0))";

        $H_kat_query = "select distinct on(k_ord) * from (select case when a.ordnung is null then k.ordnung else a.ordnung end as k_ord,
            non_empty(i.$lang,i.$lang2) as index_text,i.ordnung as index_ordnung,
            case when i.target_blank then ' target=\"_blank\"' end as target, i.id as id_index, k.id,
            k.link,non_empty(non_empty(a.$lang,a.$lang2),k.$lang) as text,f.name||i.qs as name,
            non_empty(non_empty(i.url_$lang,i.url_$lang2),'/$ls_link/$lang/'||f.name||i.qs) as url,
            a.exclude_from_top_navi
            from
            index_files i left outer join file f on f.id=i.id_file,kategorie k left outer join
            (select * from kat_aliases where id_lehr=$row1[id]) a on k.id=a.id_kat where i.id_lehr=$row1[id] and i.id_super=k.id and k.id!=1000 $H_kat_query ) a
            order by k_ord, index_ordnung desc, index_text";

        if ($this->context['IP_AUTH_OK'] ?? false)
            $H_kat_query_extern = $H_kat_query;
        else
            $H_kat_query_extern = "select distinct on(k_ord) * from (select case when a.ordnung is null then k.ordnung else a.ordnung end as k_ord,
                  non_empty(i.$lang,i.$lang2) as index_text,i.ordnung as index_ordnung,
                  case when i.target_blank then ' target=\"_blank\"' end as target, i.id as id_index, k.id,
                  k.link,non_empty(non_empty(a.$lang,a.$lang2),k.$lang) as text,f.name||i.qs as name,
                  non_empty(non_empty(i.url_$lang,i.url_$lang2),'/$ls_link/$lang/'||f.name||i.qs) as url,
                  a.exclude_from_top_navi
                  from
                  index_files i left outer join file f on f.id=i.id_file,kategorie k left outer join
                  (select * from kat_aliases where id_lehr=$row1[id]) a on k.id=a.id_kat where i.id_lehr=$row1[id] and i.id_super=k.id and k.id!=1000
                   and (k.min_power=0 and (a.min_power is null or a.min_power=0))) a
                  order by k_ord, index_ordnung desc, index_text";
        if ($this->context['AUTH_OK'] ?? false)
            $H_kat_query_intern = "select distinct on(k_ord) * from (select case when a.ordnung is null then k.ordnung else a.ordnung end as k_ord,
                  non_empty(i.$lang,i.$lang2) as index_text,i.ordnung as index_ordnung,
                  case when i.target_blank then ' target=\"_blank\"' end as target, i.id as id_index, k.id,
                  k.link,non_empty(non_empty(a.$lang,a.$lang2),k.$lang) as text,f.name||i.qs as name,
                  non_empty(non_empty(i.url_$lang,i.url_$lang2),'/$ls_link/$lang/'||f.name||i.qs) as url,
                  a.exclude_from_top_navi
                  from
                  index_files i left outer join file f on f.id=i.id_file,kategorie k left outer join
                  (select * from kat_aliases where id_lehr=$row1[id]) a on k.id=a.id_kat where i.id_lehr=$row1[id] and i.id_super=k.id and k.id!=1000
                   and
          (((
          (k.min_power<=$row1[power] and k.min_power>0 and (a.min_power is null or a.min_power<=$row1[power]))
          or
          (k.min_power=0 and a.min_power>0 and a.min_power<=$row1[power])))
          or ((k.min_power>$row1[power] or a.min_power>$row1[power])
          and hat_zugang($row1[id],$row1[id_benutzer],k.id)))) a
                  order by k_ord, index_ordnung desc, index_text";
        else
            $H_kat_query_intern = '';

        $this->context['H_kat_query'] = $H_kat_query;
        $this->context['H_kat_query_intern'] = $H_kat_query_intern;
        $this->context['H_kat_query_extern'] = $H_kat_query_extern;

        return [$H_kat_query, $H_kat_query_extern, $H_kat_query_intern];
    }

    public function setSystemUser($set = true)
    {
        if ($set)
            $this->context['row1']['id_benutzer'] = 5002;
        else if ($this->context['row1']['id_benutzer'] == 5002)
            unset($this->context['row1']['id_benutzer']);

    }

    public function hasModule($module)
    {
        $res = pg_query_params(
            $this->getDbConn(),
            'select id from modul where uname=$1',
            [$module]
        );
        return pg_num_rows($res);
    }

    public function getModConfig($module, $unames)
    {
        if (!is_array($unames))
            $unames = [$unames];

        $v = [
            $this->context['row1']['id'],
            $module
        ];
        $nr = 3;
        $query = '';
        foreach ($unames as $uname) {
            $query .= ' or mdc.uname ilike $' . $nr;
            $v[] = $uname;
            $nr++;
        }

        $res = pg_query_params(
            $this->getDbConn(),
            'select mdc.uname, case when mlc.value is null then mdc.value else mlc.value end as value
        from modul_default_config mdc left outer join 
        (select * from modul_ls_config where id_lehr=$1) mlc on mdc.id=mlc.id_modconfig
        where mdc.mod=$2 and (false ' . $query . ')',
            $v
        );
        $config = [];
        for ($i = 0; $i < pg_num_rows($res); $i++) {
            $r = pg_fetch_row($res, $i);
            $config[$r[0]] = $r[1];
        }
        return $config;
    }


    public function checkObject(int $id)
    {
        $obj = new \BayCMS\Base\BayCMSObject($this);
        try {
            $obj->load($_GET['id_obj']);
        } catch (\Exception $e) {
            $this->_404($e->getMessage());
        }

        $obj->checkReadAccess(); //Will redirect, if object does not belong to unit.

        if (!isset($this->context['ADDITIONAL_HTML_HEAD']))
            $this->context['ADDITIONAL_HTML_HEAD'] = '';
        $this->context['ADDITIONAL_HTML_HEAD'] .= $obj->getHTMLHeader();
        $values = $obj->get();
        $title = $values[$this->context['lang'] ?? 'en'];
        if (!$title)
            $title = $values[$this->context['lang2'] ?? 'de'];

        $dt = new \DateTime();
        $dt->setTimestamp($values['utime']);
        $this->context['last_modified'] = $dt->format("d.m.Y H:i");
        $this->context['title'] .= ": " . $title;
        $this->context['object_title'] = $title;
        return ': ' . $title;
    }

    public function checkHost()
    {
        if ($this->get('min_power') && isset($_SERVER['HTTPS'])) {
            $protokoll = "https";
            $httphost = $this->context['row1']['httpshost'];
        } else {
            $protokoll = "http";
            $httphost = $this->context['row1']['httphost'];
        }
        if ($httphost && $httphost != $_SERVER['HTTP_HOST'] && !isset($header_no_domain_redirect)) {
            header("Location: $protokoll://$httphost" . $_SERVER['PHP_SELF'] . ($_SERVER['QUERY_STRING'] ? "?$_SERVER[QUERY_STRING]" : ""));
            exit();
        }
    }

    public function getLoginLogoutLinks()
    {
        if ($this->get('AUTH_OK'))
            $H_LoginLogout = [
                'url' => "/" . $this->getLsLinkLang() . "/intern/gru/index.php?aktion=logout",
                'text' => 'LOGOUT'
            ];
        else {
            $H_LoginLogout = [
                'url' => ($_SERVER['HTTP_HOST'] == "localhost" || (isset($NOSSL) && $NOSSL) ? "" :
                    "https://" . ($this->get('row1', 'httpshost') ? $this->get('row1', 'httpshost') :
                        "www.bayceer.uni-bayreuth.de")),
                'text' => 'LOGIN'
            ];

            $H_LoginLogout['url'] .= ($this->context['row1']['loginlogout'] == 't' ?
                "/" . $this->getLsLinkLang() . "/top/gru/login.php" :
                "/" . $this->getLsLinkLang() . "/intern/gru/index.php?force_login=1");
        }
        $this->context['H_LoginLogout'] = $H_LoginLogout;
        return $H_LoginLogout;
    }

    public function getTemplateFile($file): bool|string
    {
        $templates = explode(".", $this->context['row1']['style']);
        for ($i = count($templates); $i--; $i >= 0) {
            $check_file = $this->context['BayCMSRoot'] . "/template/" . implode(".", $templates) . "/$file";
            if (is_readable($check_file))
                return $check_file;
            unset($templates[$i]);
        }
        return false;
    }

    public function init()
    {
        $this->checkMinPower();
        $this->checkHost();
        $auth = new \BayCMS\Base\BayCMSAuthenticator($this);
        $auth->authenticate();
        $no_frame = 0;
        $frameable = 0;
        if (isset($_GET['no_frame']))
            $no_frame = $_GET['no_frame'];
        if (isset($_GET['js_select']) && in_array($_GET['js_select'], array('n', '1', 'tiny'))) {
            $no_frame = 1;
            $frameable = 1;
        }
        $this->context['no_frame'] = $no_frame;
        $this->context['frameable'] = $frameable;
    }


    public function initTemplate()
    {
        if ($TE_INC = $this->getTemplateFile("TE.php"))
            require_once $TE_INC;
        else
            require_once $this->context['BayCMSRoot'] . "/template/bootstrap/TE.php";
    }

    public function printHeader()
    {
        $this->getIndexQueries();
        $this->getLoginLogoutLinks();
        $this->getPrePostHtml();
        if ($this->get('min_power') < 500 && isset($_GET['id_obj']) && $_GET['id_obj'] > 5000) {
            $this->checkObject($_GET['id_obj']);
        }
        $this->initTemplate();
        $GLOBALS['TE']->printHeader();
        if (!$this->context['no_frame'])
            echo $GLOBALS['TE']->htmlPostprocess($this->get('page_prefix', null, ''));
    }


    public function printFooter()
    {
        if (!$this->context['no_frame'])
            echo $GLOBALS['TE']->htmlPostprocess($this->get('page_postfix', null, ''));
        $GLOBALS['TE']->printFooter();
        exit();
    }



    public function getRow1String($prefix)
    {
        $res = $this->get('row1', $prefix . $this->get('lang'));
        if (!$res)
            $res = $this->get('row1', $prefix . $this->get('lang2'));
        return $res;
    }

    public function getLangLang2($prefix)
    {
        return $prefix . $this->get('lang') . ',' . $prefix . $this->get('lang2');
    }

    public function getLsLinkLang()
    {
        return $this->context['ls_link'] . '/' . $this->context['lang'];
    }

    public function getLsLogo($what)
    {
        if ($this->get('row1', 'logo'))
            return '';
        return '/' . $this->get('ls_link') . '/de/file/logo' . $this->get('row1', 'id') . '.' . $this->get('row1', 'logo');
    }
}