<?php

namespace BayCMS\Base;
use Exception;

class BayCMSFile extends BayCMSObject
{

    public static function replaceLocalDe(string $s)
    {
        $trans = [
            'ä' => 'ae',
            'ö' => 'oe',
            'ü' => 'ue',
            'Ä' => 'Ae',
            'Ö' => 'Oe',
            'Ü' => 'Ue',
            'ß' => 'ss',
            ' ' => '_',
            '%' => '',
            '?' => '_',
            '&' => '',
            '=' => '',
            ';' => '',
            '"' => '',
            "'" => '',
            '/' => '',
            '\\' => ''
        ];
        return strtr($s, $trans);
    }


    private ?string $source = '';
    private string $descripton = '';
    private string $path; //relativ to $BayEOSRoot 
    private string $full_path;
    private string $name;
    private int $extract = 0;
    private int $add_id_obj = 0;

    public function get(): array
    {
        $res = parent::get();
        $res['name'] = $this->name;
        $res['description'] = $this->descripton;
        $res['path'] = $this->path;
        $res['full_path'] = $this->full_path;
        $res['add_id_obj'] = $this->add_id_obj;
        $res['source'] = $this->source;
        return $res;
    }

    public function load(int $id = null)
    {
        parent::load($id);
        $res = pg_query_params(
            $this->context->getRwDbConn(),
            'select name,beschreibung from file where id=$1',
            [$this->id]
        );
        if (!pg_num_rows($res))
            throw new \BayCMS\Exception\notFound("File with id=" . $this->id . " does not exist\n");
        [$this->name, $this->descripton] = pg_fetch_row($res, 0);
        if (!preg_match('|^(.*)/([^/]+)$|', $this->name, $matches))
            preg_match('|(.*)/([^/]+/)$|', $this->name, $matches);
        $this->name = $matches[2];
        $this->full_path = $matches[1];
        if (preg_match('|(/?[0-9]*)/(' . $id . ')$|', $this->full_path, $matches)) {
            $this->add_id_obj = 1;
            $this->path = preg_replace('|' . $matches[1] . '/' . $matches[2] . '|', '', $this->full_path);
        } else {
            $this->path = $this->full_path;
            $this->add_id_obj = 0;
        }
    }

    public function save($start_transaction = true): int
    {
        $this->name = $this->replaceLocalDe($this->name);
        if ($this->extract && preg_match('/(.*)\.zip$/', $this->name, $match)) {
            $this->name = $match[1] . '/';
        } else
            $this->extract = 0;
        preg_replace("|^/|", '', $this->path);
        if (!preg_match("|/$|", $this->path))
            $this->path .= "/";
        if (!preg_match('|^[a-zA-Z0-9]|', $this->path) || preg_match('|\\.\\./|', $this->path)) {
            throw new \BayCMS\Exception\invalidPath("Invalid path '" . $this->path . "'");
        }

        if ($this->source && !is_readable($this->source)) {
            throw new \BayCMS\Exception\fileNotReadable('Source is not readable');
        }

        if ($this->source) {
            $new_source = tempnam($this->context->get('BayCMSRoot') . "/tmp", 'upload');
            if (move_uploaded_file($this->source, $new_source))
                $this->source = $new_source;
            else
                unlink($new_source);
            if (is_writable($this->source))
                chmod($this->source, 0644);
        }

        if ($this->id === null) {
            if (!$this->source || !is_readable($this->source)) {
                throw new \BayCMS\Exception\missingData('Cannot save a file without a source');
            }

            $res = pg_query_params(
                $this->context->getRwDbConn(),
                'select file_save($1,$2,$3,$4,$5,$6,$7,$8,$9,$10,$11,$12)',
                [
                    $this->context->get('row1', 'id_benutzer'),
                    $this->context->get('row1', 'id'),
                    $this->source,
                    $this->name,
                    $this->path,
                    $this->de,
                    $this->en,
                    $this->descripton,
                    $this->id_parent,
                    $this->add_id_obj,
                    0,
                    $this->extract
                ]
            );
            [$this->id] = pg_fetch_row($res, 0);
            $this->cleanUp();
            return $this->id;
        }


        $mtime = 'now()';
        if (!$this->source) {
            $this->source = null;
            $mtime = 'mtime';
        }
        pg_query_params(
            $this->context->getRwDbConn(),
            'select file_update($1,$2,$3,$4,$5,$6,$7,$8,$9,$10,$11,$12,$13,' . $mtime . ')
         from file where id=$3',
            [
                $this->context->get('row1', 'id_benutzer'),
                $this->context->get('row1', 'id'),
                $this->id,
                $this->source,
                $this->name,
                $this->path,
                $this->de,
                $this->en,
                $this->descripton,
                $this->id_parent,
                $this->add_id_obj,
                0,
                $this->extract
            ]
        );
        $this->cleanUp();
        return $this->id;

    }


    private function cleanUp()
    {
        pg_query_params(
            $this->context->getRwDbConn(),
            'update file set size=c_file_size($2||name) where id=$1',
            [$this->id, $this->context->get('BayCMSRoot') . '/']
        );
        if (
            is_writable($this->source) && ($_SERVER['REMOTE_ADDR'] ?? false) &&
            preg_match('|/tmp/|', $this->source)
        )
            unlink($this->source);

        $this->load($this->id);
    }

    public function set(array $values)
    {
        parent::set($values);
        if (isset($values['source']))
            $this->source = $values['source'];
        if (isset($values['name']))
            $this->name = $values['name'];
        if (isset($values['path']))
            $this->path = $values['path'];
        if (isset($values['description']))
            $this->descripton = $values['description'];
        if (isset($values['extract']))
            $this->extract = $values['extract'];
        if (isset($values['add_id_obj']))
            $this->add_id_obj = $values['add_id_obj'];
    }

}