<?php

namespace BayCMS\Field;

class BilangInput extends Field
{
    protected \BayCMS\Fieldset\Fieldset $fields;
    private static array $lang = ['de' => 'deutsch', 'en' => 'english'];
    public function __construct(\BayCMS\Base\BayCMSContext $context, array $settings)
    {
        $this->context = $context;
        if (!isset($settings['name'])) {
            throw new \Exception('Cannot create a field without name');
        }
        if (!isset($settings['description']))
            $settings['description'] = $settings['name'];

        $this->fields = new \BayCMS\Fieldset\Fieldset();

        $class = '\\' . __NAMESPACE__ . '\\' . ($settings['field_class'] ?? 'TextInput');
        foreach (BilangInput::$lang as $key => $value) {
            $new_settings = $settings;
            $new_settings['description'] .= ' (' . $value . ')';
            $new_settings['name'] .= $key;
            $this->fields->addField(new $class($context, $new_settings));
        }

        $this->name = strtolower($settings['name']);
        $this->settings = $settings;

    }

    public function getFields(): array
    {
        return $this->fields->getFields();
    }

    public function getValue()
    {
        $value = [];
        foreach (BilangInput::$lang as $key => $v) {
            $value[$this->name . $key] = $this->fields->getField($this->name . $key)->getValue();
        }
        return $value;
    }

    public function setValue($value): bool
    {
        return $this->error;
    }


    public function setValueFromArray(array &$v, int $id=null): bool
    {
        $this->error = false;
        $v2 = '';
        foreach ($this->fields->getFields() as $f) {
            $f->setValueFromArray($v, $id);
            $v2 .= $f->getValue();
        }
        if (($this->settings['non_empty'] ?? false) && !$v2)
            $this->error = true;
        foreach ($this->fields->getFields() as $f) {
            $f->setError($this->error);
        }
        return $this->error;

    }


    public function getSQL($prefix = 't.')
    {
        $name = $this->getName();
        $name = $prefix . $name;
        return "non_empty(" . $name . $this->context->get('lang') . "," . $name . $this->context->get('lang2') . ")";
    }

    public function getDisplayValue(): string
    {
        $v = $this->fields->getField($this->name . $this->context->get('lang'))->getDisplayValue();
        if (!$v)
            $v = $this->fields->getField($this->name . $this->context->get('lang2'))->getDisplayValue();
        return htmlspecialchars($v);
    }


    public function getInput(\BayCMS\Fieldset\Form $form): string
    {
        return '';
    }

    public function getFormRow(\BayCMS\Fieldset\Form $form): string
    {
        $out = "";
        foreach ($this->fields->getFields() as $f) {
            $out .= $f->getFormRow($form);
        }
        return $out;
    }


}