<?php

namespace BayCMS\Field;

class Date extends TextInput
{
    public function __construct(\BayCMS\Base\BayCMSContext $context, array $settings)
    {
        $options = $settings['input_options'] ?? '';
        $settings['input_options'] = $options . ' type="date"';
        parent::__construct($context, $settings);
    }

    public function getDisplayValue(): string
    {
        if (!$this->value)
            return '';
        $dt = new \DateTime($this->value);
        if ($this->context->get('lang') == 'en')
            return $dt->format('Y-m-d');
        else
            return $dt->format('d.m.Y');
    }

    public function getValue()
    {
        if ($this->value !== null && $this->value > '')
            return $this->value;
        return null;
    }

    public function setValue($value): bool
    {
        if (!$value)
            $value = null;
        if ($value) {
            $dt = new \DateTime($value);
            $value = $dt->format('Y-m-d');
        }
        return parent::setValue($value);
    }

    public function getSQL($prefix = '')
    {
        if ($this->get('sql'))
            return $this->get('sql');
        if ($this->context->get('lang') == 'de')
            return "to_char($prefix" . $this->getName() . ",'DD.MM.YYYY')";
        return parent::getSQL($prefix);
    }

}