<?php

namespace BayCMS\Field;

use function PHPUnit\Framework\returnSelf;

class Datetime extends TextInput
{
    public function __construct(\BayCMS\Base\BayCMSContext $context, array $settings)
    {
        $options = $settings['input_options'] ?? '';
        if (isset($settings['step']))
            $options .= ' step="' . $settings['step'] . '"';
        $settings['input_options'] = $options . ' type="datetime-local"';
        parent::__construct($context, $settings);
    }

    public function getDisplayValue(): string
    {
        if (!$this->value)
            return '';
        $dt = new \DateTime($this->value);
        if ($this->context->get('lang') == 'en')
            return $dt->format('Y-m-d H:i');
        else
            return $dt->format('d.m.Y H:i');
    }

    public function getValue()
    {
        if ($this->value !== null && $this->value > '')
            return $this->value;
        return null;
    }

    public function setValue($value): bool
    {
        if ($value) {
            if (!isset($this->settings['step']) || $this->settings['step'] == 60)
                $format = 'Y-m-d H:i';
            else
                $format = 'Y-m-d H:i:s';
            $dt = new \DateTime($value);
            $value = $dt->format($format);
        } else
            $value = null;
        return parent::setValue($value);
    }

    public function getSQL($prefix = '')
    {
        if ($this->get('sql'))
            return $this->get('sql');
        if ($this->context->get('lang') == 'de')
            return "to_char($prefix" . $this->getName() . ",'DD.MM.YYYY HH24:MI')";
        return "to_char($prefix" . $this->getName() . ",'YYYY-MM-DD HH24:MI')";
    }

}