<?php

namespace BayCMS\Field;


/**
 * Select
 * 
 * Settings:
 * values, db_query:
 * You have to give either "values" in the form
 * [ key1, key2, ..]
 * or
 * [ [key1, description1, group1 ],
 *   [key2, description2, group2],
 * ... ]
 * 
 * or db_query in the form
 * select xx as id, yy as description, zz as group from table where ...
 * 
 * null: adds null value to Select
 * 
 * 
 * 
 */
class Select extends TextInput
{
    public function getInput(\BayCMS\Fieldset\Form $form): string
    {
        $out = "<select id=\"" . $this->getID($form) . "\" name=\"" . $this->name . "\" ";
        $out .= ($this->settings['input_options'] ?? '') . ">\n";
        if ($this->settings['null'] ?? false)
            $out .= '<option value="null">&nbsp;</option>\n';
        if (!isset($this->settings['values'])) {
            if (!isset($this->settings['db_query']))
                throw new \BayCMS\Exception\missingData("no values and no db_query");
            $res = pg_query($this->context->getDbConn(), $this->settings['db_query']);
            $this->settings['values'] = [];
            for ($i = 0; $i < pg_num_rows($res); $i++) {
                $r = pg_fetch_row($res, $i);
                $this->settings['values'][] = $r;
            }
        }
        $group = '';
        foreach ($this->settings['values'] as $v) {
            if(! is_array($v)) $v=[$v];
            if ($group != ($v[2] ?? '')) {
                if ($group)
                    $out .= "</optgroup>\n";
                $group = $v[2];
                $out .= '<optgroup label="' . $group . '">' . "\n";
            }
            $out .= '<option value="' . htmlspecialchars($v[0]) . '"';
            if ($this->value!==null && $this->value == $v[0])
                $out .= " selected";
            $out.='>'.($v[1]??$v[0])."</option>\n";
        }
        if ($group)
                    $out .= "</optgroup>\n";
        $out.="</select>\n";
        return $out;
    }

    public function getValue()
    {
        if ($this->value !== null)
            return $this->value;
        return null;
    }

    public function setValue($value): bool
    {
        if($value==="null") $value=null;
        $this->error = false;
        if (($this->settings['non_empty'] ?? false) && $value===null)
            $this->error = true;
        $this->value = $value;
        return $this->error;
    }
    public function getDisplayValue(): string{
        if(! $this->value) return '';
        $value=$this->value;
        if(isset($this->settings['values'])){
            foreach($this->settings['values'] as $v){
                if(! is_array($v)) $v=[$v];
                if($v[0]==$this->value){
                    $value=$v[1]??$v[0];
                    break;
                }
            }
        }
        if(isset($this->settings['db_query']) && $this->value!==null ){
            $res=pg_query_params($this->context->getDbConn(),
            'select description from ('.$this->settings['db_query'].') a where value=$1',
            [$this->value]);
            if(pg_num_rows($res)) [$value]=pg_fetch_row($res,0);
            else $value='';
        }
        return htmlspecialchars($value);
    }

}