<?php

namespace BayCMS\Field;

class SelectJSMulti extends Select
{
    public function __construct(\BayCMS\Base\BayCMSContext $context, array $settings)
    {
        if(! isset($settings['db_query'])) throw new \BayCMS\Exception\missingData("You have to set db_query");
        if (!isset($settings['no_add_to_query']))
            $settings['no_add_to_query'] = true;
        parent::__construct($context, $settings);
    }


    public function getInput(\BayCMS\Fieldset\Form $form): string
    {
        $out = "";
        // DB-Anfrage db_query mit Platzhalter $value$
        if (!isset($this->settings['input_options']))
            $this->settings['input_options'] = " rows=\"5\" cols=\"35\" wrap=\"virtual\"";

        $target = $this->settings['target'];

        $frame = $this->settings['frame'] ?? '';
        if (!$frame)
            $frame = "../../intern/gru/js_frame.php";
        if (!strstr($frame, '?'))
            $frame .= "?js_phpfile=$target";
        if ($this->settings['sorted'] ?? false)
            $frame .= "&sorted=1";

        $name = $this->name;
        if (!isset($this->settings['chooser_options']))
            $this->settings['chooser_options'] = "toolbar=no,menubar=no,scrollbars=yes,width=700,height=500";

        $out .= '<input type="hidden" name="' . $name . '" 
        value="' . htmlspecialchars($this->value) . '">';
        $out .= '
        <textarea id="' . $this->getID($form) . '" name="' . $name . '_dp" readonly' .
            ($this->settings['input_options'] ?? '') . '>';
        $out .= str_replace(", ", "\n", $this->getDisplayValue());
        $out .= "</textarea>\n";
        $out .= "
        <input type=button onClick='i" . $name . "=document." . $form->getName() .
            "." . $name . ";i" . $name . "_dp=document." .
            $form->getName() . '.' . $name . "_dp; " . $name .
            "_chooser=window.open(\"" . $frame . "&target=i" . $name . '",
          "' . $name . "_chooser\", \"" .
            $this->settings['chooser_options'] . '"); ' . $name .
            "_chooser.i" . $name . "=i" . $name . "; " . $name . "_chooser.i" .
            $name . "_dp=i" . $name . "_dp; 
        window.i$name=i$name; window.i" . $name . "_dp=i" . $name . "_dp' value=\"...\">";
        return $out;

    }

    public function getDisplayValue(): string
    {
        if (!$this->value)
            return '';

        $in = [];
        $val = preg_split("/, ?/", $this->value);

        for ($i = 1; $i <= count($val); $i++) {
            $in[] = '$' . $i;
        }
        $in = implode(',', $in);
        $res = pg_query_params(
            $this->context->getDbConn(),
            str_replace('${in}', $in, $this->settings['db_query']),
            $val
        );

        $out = [];
        for ($i = 0; $i < pg_num_rows($res); $i++) {
            $r = pg_fetch_row($res, $i);
            $out[$r[0]] = htmlspecialchars($r[1]);
        }
        $out2 = [];
        foreach ($val as $v) {
            if (!isset($out[$v]))
                continue;
            $out2[] = $out[$v];
        }
        return implode(", ", $out2);
    }


    public function setValueFromArray(array &$v, ?int $id = null): bool
    {
        if (isset($v[$this->getName()]))
            return parent::setValue($v[$this->getName()]);
        if (!$id)
            return false;

        $ordnung = $this->settings['ordnung'] ?? 1;
        $sorted = $this->settings['sorted'] ?? false;
        $limit = $this->settings['limit'] ?? 999;
        $subquery = 'select id_auf from verweis where id_von=$1 and ' .
            ($sorted ? 'ordnung>=$2 and ordnung<=($2+' . $limit . ')' : 'ordnung = $2');
        $res = pg_query_params(
            $this->context->getRwDbConn(),
            str_replace('${in}', $subquery, $this->settings['db_query']),
            [$id, $ordnung]
        );
        $v2 = [];
        for ($i = 0; $i < pg_num_rows($res); $i++) {
            $r = pg_fetch_row($res, $i);
            $v2[] = $r[0];
        }
        $this->value = implode(',', $v2);
        return false;
    }

    public function save(\BayCMS\Fieldset\Form $form): bool
    {
        if (!$form->getId())
            return false;
        $ordnung = $this->settings['ordnung'] ?? 1;
        $sorted = $this->settings['sorted'] ?? false;
        $limit = $this->settings['limit'] ?? 999;
        $conn = $this->context->getRwDbConn();

        pg_prepare($conn, $this->getName() . 'insert', 'insert into verweis(id_von, id_auf, ordnung) values ($1, $2, $3)');
        pg_prepare($conn, $this->getName() . 'delete', 'delete from verweis where id_von=$1 and id_auf=$2 and ordnung=$3');

        $query = 'select id_auf from verweis where id_von=$1 and ' .
            ($sorted ? 'ordnung>=$2 and ordnung<=($2+' . $limit . ')' : 'ordnung = $2') ;
        $query= str_replace('${in}', $query, $this->settings['db_query']);
        $query = 'select id_auf,ordnung from verweis where id_von=$1 and id_auf in (select id from ('.$query.') a ) and ' .
            ($sorted ? 'ordnung>=$2 and ordnung<=($2+' . $limit . ')' : 'ordnung = $2') . ' order by 2,1';
        
        $res = pg_query_params(
            $conn,
            $query,
            [$form->getId(), $ordnung]
        );



        $old = [];
        for ($i = 0; $i < pg_num_rows($res); $i++) {
            $r = pg_fetch_array($res, $i);
            $old[$r['id_auf']] = $r['ordnung'];
        }

        $new = [];
        if ($this->value && $ids = preg_split("/, ?/", $this->value)) {
            foreach ($ids as $id) {
                $new[$id] = $ordnung;
                if ($sorted)
                    $ordnung++;
            }
        }

        foreach ($new as $id => $ordnung) {
            if (isset($old[$id]) && $old[$id] == $ordnung) {
                unset($old[$id]);
                continue;
            }
            pg_execute($conn, $this->getName() . 'insert', [$form->getId(), $id, $ordnung]);
        }

        foreach ($old as $id => $ordnung) {
            pg_execute($conn, $this->getName() . 'delete', [$form->getId(), $id, $ordnung]);
        }
        return true;
    }

}