<?php

namespace BayCMS\Field;

class SelectRadio extends Select
{
    private array $fields = [];

    public function __construct(\BayCMS\Base\BayCMSContext $context, array $settings)
    {
        if (!isset($settings['values']) && !isset($settings['db_query']))
            throw new \BayCMS\Exception\missingData("no values and no db_query");
        parent::__construct($context, $settings);


        if (!isset($this->settings['values'])) {
            $res = pg_query($this->context->getDbConn(), $this->settings['db_query']);
            $this->settings['values'] = [];
            for ($i = 0; $i < pg_num_rows($res); $i++) {
                $r = pg_fetch_row($res, $i);
                $this->settings['values'][] = $r;
            }
        }
        foreach ($this->settings['values'] as $v) {
            $this->fields[] = new Radio($context, [
                'name' => $this->name,
                'value' => $v[0],
                'description' => $v[1],
                'default_value' => $this->settings['default_value']
            ]);
        }
        if ($this->settings['free_input'] ?? false) {
            $this->fields[] = new TextInput($context, [
                'name' => $this->name . '_free',
                'description' => $this->settings['free_input']
            ]);
        }

    }


    public function setValueFromArray(array &$v, ?int $id = null): bool
    {
        if ($v[$this->name . '_free'] ?? false) {
            return $this->setValue($v[$this->name . '_free']);
        }
        return $this->setValue($v[$this->name] ?? null);
    }

    public function setValue($value): bool
    {
        $this->error = 0;
        foreach ($this->fields as $f) {
            $f->setValue($value);
            if ($value == $f->getValue()) {
                break;
            }
        }
        if (!$value && ($this->settings['non_empty'] ?? false)) {
            $this->error = 1;
        }
        foreach ($this->fields as $f) {
            $f->setError($this->error);
        }
        $this->value = $value;
        return $this->error;
    }

    public function getInput(\BayCMS\Fieldset\Form $form): string
    {
        return '';
    }


    public function getDisplayValue(): string
    {
        $display_value=parent::getDisplayValue();
        if(! $display_value) $display_value=htmlspecialchars($this->value);
        return $display_value;
    }

    public function getFormRow(\BayCMS\Fieldset\Form $form): string
    {
        $out = '<div class="' . $GLOBALS['TE']->getCSSClass('form_div') . '">';
        $out .= '<label';
        if ($this->settings['label_css'] ?? '')
            $out .= ' style="' . $this->settings['label_css'] . '"';
        $out .= '>' . $this->getDescription() . $this->getHelp($form) . '</label>' . "<br/>\n";
        $i = 0;
        foreach ($this->fields as $f) {
            $f->setID($form->getName() . '_' . $this->name . $i);
            $i++;
            $out .= $f->getInput($form);
            $out .= ' <label for="' . $f->getID($form) . '"';
            $out .= ' style="' . ($this->settings['label_css'] ?? 'width:auto;') . '"';
            $out .= '>' . $f->getDescription() . '</label>';
            $out .= "<br/>\n";
        }
        if ($this->settings['free_input'] ?? false) {
            $out .= "<script>$('#" . $f->getID($form) . "').on('click', function(){
            $('input[name=\"" . $this->getName() . "\"]').prop('checked', false)
        })</script>";
        }

        $out .= "</div>\n";
        return $out;
    }

}