<?php

namespace BayCMS\Field;

use function PHPUnit\Framework\returnSelf;

class Textarea extends TextInput
{

    public function getInput(\BayCMS\Fieldset\Form $form): string
    {
        $out = "<textarea id=\"" . $this->getID($form) . "\" name=\"" . $this->name . "\" ";
        if ($this->settings['placeholder'] ?? '')
            $out .= 'placeholder="' . htmlspecialchars($this->settings['placeholder']) . '"';

        $out .= ($this->settings['input_options'] ?? ' rows="5" cols="35" wrap="virtual"') . ">";
        $out .= htmlspecialchars($this->value);
        $out .= '</textarea>' . "\n";
        if ($this->settings['htmleditor'] ?? false)
            $out .= "<a href=\"#\" onclick='itext = document.forms[\"" . $form->getName() . "\"].elements[\"" . $this->settings['name'] . "\"]; 
        editor = window.open(\"/" . $this->context->get('ls_link') . '/' . $this->context->get('lang') . "/intern/gru/editor.php?id_obj=" . $form->getId() . "\", 
        \"EDITOR\", \"toolbar=no,menubar=no,scrollbars=yes,width=700,height=500\"); 
        editor.itext = itext; window.itext = itext;'>...HTML-Editor</a>\n";

        return $out;
    }

    public function save(\BayCMS\Fieldset\Form $form): bool
    {
        if (!$form->get('uname'))
            return true;
        $matches = [];
        preg_match_all(
            "&src=\"[^\"]+(/|=)[ot]?([0-9]+)\\.(jpg|gif|png|svg)\"&",
            $this->value,
            $matches,
            PREG_PATTERN_ORDER
        );

        $res = pg_query_params(
            $this->context->getRwDbConn(),
            'select id_bild from objekt_bild where id_obj=$1 and name=$2',
            [$form->getId(), $this->getName()]
        );
        $del = [];
        for ($i = 0; $i < pg_num_rows($res); $i++) {
            [$id] = pg_fetch_row($res, $i);
            $index=array_search($id, $matches[2]);
            if ($index !== false) {
                unset($matches[2][$index]);
            } else {
                $del[] = $id;
            }
        }
        if (count($matches[2])) {
            pg_prepare(
                $this->context->getRwDbConn(),
                $this->getName() . 'insert_objekt_bild',
                'insert into objekt_bild(id_obj,id_bild,name) values($1,$2,$3)'
            );
            foreach ($matches[2] as $id) {
                pg_execute(
                    $this->context->getRwDbConn(),
                    $this->getName() . 'insert_objekt_bild',
                    [$form->getId(), $id, $this->getName()]
                );
            }
        }

        if (count($del)) {
            pg_prepare(
                $this->context->getRwDbConn(),
                $this->getName() . 'delete_objekt_bild',
                'delete from objekt_bild where id_obj=$1 and id_bild=$2 and name=$3'
            );
            foreach ($del as $id) {
                pg_execute(
                    $this->context->getRwDbConn(),
                    $this->getName() . 'delete_objekt_bild',
                    [$form->getId(), $id, $this->getName()]
                );
            }

        }
        pg_query_params(
            $this->context->getRwDbConn(),
            'delete from objekt_bild where id_obj=$1 and name is null',
            [$form->getId()]
        );

        return true;


    }

    public function getDisplayValue(): string
    {
        $v = $this->value;
        $max = ($this->settings['max_display_length'] ?? 300);
        if (strlen($v) > $max) {
            $v = substr($v, 0, $max) . "...";
        }
        return htmlspecialchars($v);
    }

}