<?php

namespace BayCMS\Field;


class Upload extends File
{

    protected \BayCMS\Fieldset\Fieldset $fields;
    public function __construct(\BayCMS\Base\BayCMSContext $context, array $settings)
    {
        parent::__construct($context, $settings);
        $this->fields = new \BayCMS\Fieldset\Fieldset();
        $this->fields->addField(new Hidden($context, ['name' => $this->getName() . '_location']));
        $this->fields->addField(new Hidden($context, ['name' => $this->getName() . '_name']));
        $this->fields->addField(new Checkbox($context, ['name' => $this->getName() . '_del', 'description' => $this->t('delete file', 'Datei löschen')]));
    }

    public function setValueFromArray(array &$v, ?int $id = null): bool
    {
        $this->error = false;
        $this->inline_error='';
        if(! isset($v[$this->getName().'_name'])) return $this->error; //not called with $_POST

        if ($_FILES[$this->getName()]['tmp_name'] ?? false) {
            if (is_writable($_POST[$this->getName() . '_location'] ?? false))
                unlink($_POST[$this->getName() . '_location']);

            $this->fields->getField($this->getName() . '_location')->setValue(null);
            $this->fields->getField($this->getName() . '_name')->setValue(null);
            if (
                $this->get('preg_match') &&
                !preg_match($this->get('preg_match'), $_FILES[$this->getName()]['name'])
            ) {
                $this->error = true;
                $this->inline_error=$this->t('Not accepted file type','Nicht akzeptierter Dateityp').': '.htmlspecialchars($_FILES[$this->getName()]['name']);
                return $this->error;
            }
            $newname = tempnam($this->context->get('BayCMSRoot') . "/tmp", session_id());
            move_uploaded_file($_FILES[$this->getName()]['tmp_name'], $newname);
            $this->fields->getField($this->getName() . '_location')->setValue($newname);
            $this->fields->getField($this->getName() . '_name')->setValue($_FILES[$this->getName()]['name']);
        } elseif ($_POST[$this->getName() . '_del'] ?? false) {
            if (is_writable($_POST[$this->getName() . '_location'] ?? false))
                unlink($_POST[$this->getName() . '_location']);

            $this->fields->getField($this->getName() . '_location')->setValue(null);
            $this->fields->getField($this->getName() . '_name')->setValue(null);
        } elseif ($_POST[$this->getName() . '_name'] ?? false) {
            $this->fields->getField($this->getName() . '_location')->setValue($_POST[$this->getName() . '_location'] ?? null);
            $this->fields->getField($this->getName() . '_name')->setValue($_POST[$this->getName() . '_name']);
            if (!is_readable($_POST[$this->getName() . '_location'] ?? null)) {
                //File does not exist any more on the server....
                $this->fields->getField($this->getName() . '_location')->setValue(null);
                $this->fields->getField($this->getName() . '_name')->setValue(null);
            }
        }
        if ($this->get('non_empty') && !$this->fields->getField($this->getName() . '_location')->getValue())
            $this->error = true;
        return $this->error;
    }

    public function getInput(\BayCMS\Fieldset\Form $form): string
    {
        $out = parent::getInput($form);
        $out .= $this->fields->getField($this->getName() . '_name')->getInput($form);
        $out .= $this->fields->getField($this->getName() . '_location')->getInput($form);

        $location = $this->fields->getField($this->getName() . '_location')->getValue();
        $name = $this->fields->getField($this->getName() . '_name')->getValue();
        if ($location) {
            $res = pg_query_params(
                $this->context->getDbConn(),
                'select get_filetype_image($1)',
                [$name]
            );
            [$img] = pg_fetch_row($res, 0);
            $size = round(filesize($location) / 1024);
            $out .= "<br/><a href=\"/" . $this->context->get('ls_link') . "/" . $this->context->get('lang') . "/top/gru/get.php?f=";
            $out .= urlencode($location);
            $out .= "&n=";
            $out .= urlencode($name);
            $out .= "\" target=\"_blank\">$img $name ($size kb)</a> ";
            $f = $this->fields->getField($this->getName() . '_del');
            $out .= $f->getInput($form);
            $out .= " " . $f->getDescription();
        }
        return $out;

    }


}