<?php

namespace BayCMS\Fieldset;

use SebastianBergmann\RecursionContext\Context;

class BayCMSList extends Fieldset
{

    protected int $num_rows = -1;


    public function __construct(\BayCMS\Base\BayCMSContext $context, array $settings)
    {
        $this->context = $context;
        $defaults = [
            'from' => null,
            'where' => null,
            'prefix' => 't.',
            'step' => 20,
            'name' => 'table1',
            'id_query' => '',
            'write_access_query' => '',
            'offset' => 0,
            'qs' => '',
            'order_by' => [1],
            'auto_order' => false,
            'id_name' => 'id',
            'view_file' => '',
            'action_sep' => "<br/>\n",
            'actions' => []
        ];
        foreach ($defaults as $k => $v) {
            if (!isset($settings[$k]))
                $settings[$k] = $v;
            if (is_null($settings[$k])) {
                throw new \BayCMS\Exception\missingData("BayCMSList: $k must not be null");
            }
        }
        $this->settings = $settings;
    }



    public function createQuery()
    {
        if (isset($_GET[$this->get('name') . 'order_by']))
            $oa = explode("-", $_GET[$this->get('name') . 'order_by']);
        else
            $oa = [0, 'a'];
        $oa[0] = intval($oa[0]);
        if (!isset($oa[1]))
            $oa[1] = '';
        $order_by_query = " order by ";
        if ($this->get('auto_order')) {
            $order_by_query .= (isset($this->get('order_by')[$oa[0]]) ? $this->get('order_by')[$oa[0]] : $oa[0]);
            if ($oa[1] == "d")
                $order_by_query .= " desc";
            for ($i = 0; $i < count($this->get('order_by')); $i++) {
                if ($oa[0] != $i)
                    $order_by_query .= "," . $this->get('order_by')[$i];
            }

        } else {
            $order_by_query .= $this->get('order_by')[0];
            for ($i = 1; $i < count($this->get('order_by')); $i++) {
                $order_by_query .= "," . $this->get('order_by')[$i];
            }
        }


        $query = "select ";
        foreach ($this->fields as $field) {
            $query .= $field->getSQL($this->get('prefix')) . ", ";
        }
        if ($this->get('id_query'))
            $query .= $this->get('id_query');
        else
            $query .= "1";
        if ($this->get('write_access_query'))
            $query .= "," . $this->get('write_access_query');
        $query .= " from " . $this->get('from') . " where " . $this->get('where');
        $query .= $order_by_query;
        if ($this->get('step') > 0)
            $query .= " limit " . $this->get('step') . " offset " . $this->get('offset');
        return $query;
    }


    public function getNumRows($force = false)
    {
        if ($this->num_rows >= 0 && !$force)
            return $this->num_rows;
        $res = pg_query(
            $this->context->getDbConn(),
            'select 1 from ' . $this->get('from') . ' where ' . $this->get('where')
        );
        $this->num_rows = pg_num_rows($res);
        return $this->num_rows;
    }

    function bootstrap_pagination()
    {
        $nameoffset = $this->get('name') . "offset";
        $step = $this->get('step');
        $href = "<a href=\"?" . $this->get('qs') . "&$nameoffset=";
        $max = $this->getNumRows();
        if ($max <= $step)
            return;
        if ($step <= 0)
            return;
        $out = '<ul class="pagination">';
        $current = $this->get('offset') / $step + 1;
        if ($current < 1)
            $current = 1;
        if ($current > ceil($max / $step))
            $current = ceil($max / $step);

        $pag = array($current);
        $current_step = 1;
        $last = $current;
        $pag_count = 0;
        while (($current - $current_step) >= 1) {
            $pag_count++;
            if ($max / $step > 12 || $pag_count > 2) {
                $new = ceil(($current - $current_step) / $current_step) * $current_step;
                $current_step *= 10;
            } else {
                $new = round($current - $current_step);
                $current_step++;
                if ($pag_count > 1)
                    $current_step = 10;
            }
            if ($new != $last) {
                $pag[] = $new;
                $last = $new;
            }

        }
        $current_step = 1;
        $last = $current;
        $pag_count = 0;
        while (($current + $current_step) <= ceil($max / $step)) {
            $pag_count++;
            if ($max / $step > 12 || $pag_count > 2) {
                $new = floor(($current + $current_step) / $current_step) * $current_step;
                $current_step *= 10;
            } else {
                $new = round($current + $current_step);
                $current_step++;
                if ($pag_count > 1)
                    $current_step = 10;
            }
            if ($new != $last) {
                $pag[] = $new;
                $last = $new;
            }
        }
        sort($pag);


        $out .= '<li' . ($current > 1 ? '' : ' class="disabled"') . '>' . $href . '0' . '">&laquo;</a></li>';

        $out .= '<li' . ($current > 1 ? '' : ' class="disabled"') . '>' . $href . ($this->get('offset') - $step) . '">&lsaquo;</a></li>';
        for ($i = 0; $i < count($pag); $i++) {
            $out .= '<li' . ($current == $pag[$i] ? ' class="active"' : '') . '>' . $href . (($pag[$i] - 1) * $step) . '">' . $pag[$i] . '</a></li>';
        }
        $out .= '<li' . (($current) >= ceil($max / $step) ? ' class="disabled"' : '') . '>' . $href . ($this->get('offset') + $step) . '">&rsaquo;</a></li>';
        $out .= '<li' . (($current) >= ceil($max / $step) ? ' class="disabled"' : '') . '>' . $href . ((ceil($max / $step) - 1) * $step) . '">&raquo;</a></li>';
        $out .= '</ul>';
        return $out;

    }

    function get_nav_html()
    {
        if ($GLOBALS['TE']->isBootstrap())
            return $this->bootstrap_pagination();

        $nameoffset = $this->get('name') . "offset";
        $erste_seite = "|&laquo;&laquo;";
        $vor = "&laquo;&laquo;";
        $zurueck = "&raquo;&raquo;";
        $letzte_seite = "&raquo;&raquo;|";

        //$erste_seite="&laquo;&nbsp;".translate("erste&nbsp;Seite");
        //$vor="&laquo;";
        //$zurueck="&raquo;";
        //$letzte_seite=translate("letzte&nbsp;Seite")."&nbsp;&raquo;";


        $num = $this->getNumRows();
        if ($num <= $this->get('step') || $this->get('step') < 0) {
            $this->settings['offset'] = 0;
            $back_forward_html = "";
        } else {
            if ($this->get('offset') == 0)
                $back_forward_html = "$erste_seite&nbsp;$vor&nbsp;..&nbsp;";
            else
                $back_forward_html = "<a href=\"?$nameoffset=0&" . $this->get('qs') . "\">$erste_seite</a>&nbsp;<a href=\"?$nameoffset=" . ($this->get('offset') - $this->get('step')) . "&" . $this->get('qs') . "\">$vor</a>&nbsp;..&nbsp;";
            $step = 1;
            while ($num / $this->get('step') / $step > 10) {
                $step *= 2;
                if ($num / $this->get('step') / $step > 10)
                    $step *= 5;
            }
            for ($i = $step; $i < $num / $this->get('step'); $i += $step) {
                if ($i * $this->get('step') == $this->get('offset'))
                    $back_forward_html .= ($i * $this->get('step')) . "&nbsp;..&nbsp;";
                else
                    $back_forward_html .= "<a href=\"?$nameoffset=" . ($i * $this->get('step')) . "&" . $this->get('qs') . "\">" . ($i * $this->get('step')) . "</a>&nbsp;..&nbsp;";
            }
            if (($this->get('offset') + $this->get('step')) >= $num)
                $back_forward_html .= "$zurueck&nbsp;$letzte_seite";
            else
                $back_forward_html .= "<a href=\"?$nameoffset=" . ($this->get('offset') + $this->get('step')) . "&" . $this->get('qs') . "\">$zurueck</a>&nbsp;<a href=\"?$nameoffset=" . ($this->get('step') * floor(($num - 1) / $this->get('step'))) . "&" . $this->get('qs') . "\">$letzte_seite</a>";
            $back_forward_html .= "<br>";
        }
        return $back_forward_html;
    }

    function getAction($id, $write_access, $type)
    {
        switch ($type) {
            case "del":
                $action = $GLOBALS['TE']->getActionLink(
                    "?aktion=del&" . $this->get('id_name') . "=$id&" . $this->get('qs'),
                    $this->t('delete','löschen'),
                    " onClick=\"return confirm('" . $this->t('Do you really want to delete the row?','Zeile wirklich löschen?') . "')\"",
                    $type
                );
                if ($write_access != "f")
                    return $action;
                else
                    return "-";
            case "view":
                if (!$this->get('view_file'))
                    $this->settings['view_file'] = "?" . $this->get('qs') . '&' . $this->get('id_name') . '=';
                return $GLOBALS['TE']->getActionLink(
                    $this->get('view_file') . $id,
                    $this->t('show','ansehen'),
                    '',
                    'eye-open'
                );
            case "copy":
                return $GLOBALS['TE']->getActionLink(
                    "?aktion=copy&" . $this->get('id_name') . "=$id&" . $this->get('qs'),
                    $this->t('copy','kopieren'),
                    '',
                    $type
                );
            case "edit":
                $action = $GLOBALS['TE']->getActionLink(
                    "?aktion=edit&" . $this->get('id_name') . "=$id&" . $this->get('qs'),
                    $this->t('edit','bearbeiten'),
                    '',
                    $type
                );
                if ($write_access != "f")
                    return $action;
                else
                    return "-";
            case "edit_copy":
                return $this->getAction($id, $write_access, ($write_access != "f" ? "edit" : "copy"));
            case "properties":
                return $GLOBALS['TE']->getActionLink(
                    "../gru/objekt_detail.php?id_obj=$id",
                    $this->t('object properties','Objekt Eigenschaften'),
                    '',
                    $type
                );
        }
    }
    function getActions($id, $write_access = "t")
    {
        if (!count($this->get('actions')))
            return '';

        $out='';
        foreach($this->get('actions') as $a){
            if($out) $out .= $this->get('action_sep');
            $out .= $this->getAction($id, $write_access, $a);
        }
        $out = "<td>$out</td>";

        return $out;
    }


    public function getValues(): array
    {
        $fields = [];
        foreach ($this->fields as $f) {
            $fields[] = $f->getName();
        }        
        $fields[] = "__id";

        $res = pg_query($this->context->getDbConn(), $this->createQuery());
        $values = [];
        $num_fields=count($fields);
        for ($j = 0; $j < pg_num_rows($res); $j++) {
            $r = pg_fetch_row($res, $j);
            $values[] = array_combine($fields, array_slice($r,0,$num_fields));
        }
        return $values;
    }


    public function getJSON()
    {
        $fields = [];
        foreach ($this->fields as $f) {
            $fields[] = $f->getDescription(false);
        }
        $fields[] = "id";
        if ($this->get('write_access_query'))
            $fields[] = "write_access";
        $res = pg_query($this->context->getDbConn(), $this->createQuery());
        $json = [];
        for ($j = 0; $j < pg_num_rows($res); $j++) {
            $r = pg_fetch_row($res, $j);
            $json[] = array_combine($fields, $r);
        }
        return json_encode($json);
    }

    public function getExport($ending, $basename)
    {
        require_once("PHPExcel.php");
        // Create new PHPExcel object
        switch ($ending) {
            case 'xlsx':
                $type = 'Excel2007';
                $mime = 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet';
                break;
            case 'xls':
                $type = 'Excel5';
                $mime = 'vapplication/nd.ms-excel';
                break;
            case 'pdf':
                $type = 'PDF';
                $mime = 'application/pdf';
                break;
            case 'csv':
                $type = 'CSV';
                $mime = 'text/x-csv';
                break;
            default:
                throw new \BayCMS\Exception\unsupportedFiletype("Unsupported export format $type");
        }

        $fields = [];
        foreach ($this->fields as $f) {
            $fields[] = $f->getDescription(false);
        }
        $fields[] = "id";
        if ($this->get('write_access_query'))
            $fields[] = "write_access";

        $objPHPExcel = new \PHPExcel();
        $objPHPExcel->getProperties()->setCreator("BayCMS")
            ->setLastModifiedBy("BayCMS");
        $objPHPExcel->getActiveSheet()->fromArray($fields, NULL, 'A1');
        $first_letter = \PHPExcel_Cell::stringFromColumnIndex(0);
        $last_letter = \PHPExcel_Cell::stringFromColumnIndex(count($fields) - 1);
        $header_range = "{$first_letter}1:{$last_letter}1";
        $objPHPExcel->getActiveSheet()->getStyle($header_range)->getFont()->setBold(true);
        date_default_timezone_set('GMT');

        $res = pg_query($this->context->getDbConn(), $this->createQuery());
        for ($i = 0; $i < pg_num_rows($res); $i++) {
            $r = pg_fetch_row($res, $i);

            for ($j = 0; $j < count($fields); $j++) {
                $ct = pg_field_type($res, $j);
                if ($this->settings['export_no_strip_tags'] ?? false)
                    $value = strip_tags($r[$j]);
                else
                    $value = $r[$j];

                $cell = \PHPExcel_Cell::stringFromColumnIndex($j) . ($i + 2);
                if ($value) {
                    if ($ct == 'bool') {
                        $value = ($r[$j] == 't' ? TRUE : FALSE);
                    } elseif (strstr($ct, 'timestamp')) {
                        $t = date_parse_from_format('Y-m-d H:i:s', $r[$j]);
                        $value = \PHPExcel_Shared_Date::PHPToExcel(
                            mktime(
                                $t['hour'],
                                $t['minute'],
                                $t['second'],
                                $t['month'],
                                $t['day'],
                                $t['year']
                            )
                        );
                        $objPHPExcel->getActiveSheet()->getStyle($cell)
                            ->getNumberFormat()
                            ->setFormatCode('dd.mm.yyyy hh:mm:ss');

                    } elseif (strstr($ct, 'time')) {
                        $t = date_parse_from_format('H:i:s', $r[$j]);
                        $value = $t['hour'] / 24 + $t['minute'] / 24 / 60 + $t['second'] / 24 / 60 / 60;
                        $objPHPExcel->getActiveSheet()->getStyle($cell)
                            ->getNumberFormat()
                            ->setFormatCode('hh:mm:ss');

                    } elseif ($ct == 'date') {
                        $t = date_parse_from_format('Y-m-d', $r[$j]);
                        $value = \PHPExcel_Shared_Date::PHPToExcel(
                            mktime(0, 0, 0, $t['month'], $t['day'], $t['year'])
                        );
                        $objPHPExcel->getActiveSheet()->getStyle($cell)
                            ->getNumberFormat()
                            ->setFormatCode('dd.mm.yyyy');
                    }
                }
                $objPHPExcel->getActiveSheet()->setCellValue($cell, html_entity_decode($value, ENT_COMPAT, 'UTF-8'));


            }
        }


        // Rename worksheet
        $objPHPExcel->getActiveSheet()->setTitle('Export');


        // Set active sheet index to the first sheet, so Excel opens this as the first sheet
        $objPHPExcel->setActiveSheetIndex(0);


        // Redirect output to a client’s web browser (Excel2007)
        header('Content-Type: ' . $mime);
        header('Content-Disposition: attachment;filename="' . $basename . '.' . $ending . '"');
        header('Cache-Control: max-age=0');

        $objWriter = \PHPExcel_IOFactory::createWriter($objPHPExcel, $type);
        $objWriter->save('php://output');
        exit;

    }


    public function getTable(int $id = null)
    {
        $num = $this->getNumRows();
        $i_id = count($this->fields);
        $i_writeaccess = $i_id + 1;
        $out = '';
        if ($this->settings['action_new'] ?? false) {
            $out .= $GLOBALS['TE']->getActionLink(
                "?aktion=edit&" . $this->get('qs'),
                $this->t('New Entry', "Neuer Eintrag"),
                '',
                'new'
            );
        }
        if ($this->settings['action_upload'] ?? false) {
            if ($out)
                $out .= " &nbsp; ";
            $out .= $GLOBALS['TE']->getActionLink(
                "?aktion=upload&" . $this->get('qs'),
                $this->t('Upload Data', "Daten Hochladen"),
                '',
                'open',
                ['class' => '']
            );
        }
        if ($out)
            $out .= "<br/>\n";


        if ($this->settings['with_count'] ?? true) {
            if ($this->get('step') < 0)
                $step = $num;
            else
                $step = $this->get('step');
            $out .= "&nbsp;" . min($this->get('offset') + 1, $num) . "-" . min($this->get('offset') + $step, $num) . " " .
                $this->t('of', 'von') . " $num<br>";
        }

        $back_forward_html = $this->get_nav_html();
        $out .= $back_forward_html;
        $out .= "<table id=\"" . $this->get('name') . '" ' . $GLOBALS['TE']->getCSSClass((($settings['border'] ?? true) ? 'list_table' : 'list_table_without_border')) . '>';

        if ($this->settings['with_head'] ?? true) {
            $out .= '<tr class="list_head">';
            for ($i = 0; $i < count($this->fields); $i++) {
                $out .= '<th>' . $this->fields[$i]->getDescription(false);
                if ($this->get('auto_order') && isset($this->get('order_by')[$i])) {
                    $out.= " ";
                    if (($_GET[$this->get('name') . 'order_by'] ?? '') == "$i")
                        $out.= "<b>&darr;</b>";
                    else
                        $out.= "<a href=\"?" . $this->get('qs') . "&" . $this->get('name') . "order_by=$i\">&darr;</a>";
                    if (($_GET[$this->get('name') . 'order_by'] ?? '') == "$i-d")
                        $out.= "<b>&uarr;</b>";
                    else
                        $out.= "<a href=\"?" . $this->get('qs') . "&" . $this->get('name') . "order_by=$i-d\">&uarr;</a>";
                }

                $out .= "</th>";
            }
            for ($i = 0; $i < count($this->get('actions')); $i++) {
                if ($this->get('action_sep') != "</td><td>") {
                    $i = count($this->get('actions'));
                }
                $out .= "<th>&nbsp;</th>";
            }
            $out .= "</tr>";
        }

        if ($id && $this->get('id_query') && $num > 1) {
            //erste Zeile mit ID
            //save old settings
            $where = $this->get('where');
            $offset = $this->get('offset');
            $num_rows = $this->num_rows;
            //manually set new ons
            $this->settings['where'] .= " and " . $this->get('id_query') . "=" . $id;
            $this->settings['offset'] = 0;
            $this->num_rows = 1;

            $res = pg_query($this->context->getDbConn(), $this->createQuery());
            if (pg_num_rows($res)) {
                $r = pg_fetch_row($res, 0);
                $out .= '<tr valign="top" class="list_row_odd" data-id="' . $r[$i_id] . '">';
                for ($i = 0; $i < count($this->fields); $i++) {
                    $out .= "<td class=\"baycmslist-data-td\">$r[$i]</td>";
                }
                $out .= $this->getActions($r[$i_id], $r[$i_writeaccess] ?? 't');
                $out .= "</tr>\n";
                $cols = count($this->fields);
                if ($this->get('action_sep') != "</td><td>")
                    $cols += count($this->get('actions'));
                else
                    $cols++;
                $out .= "<tr class=\"list_row_even\"><td colspan=\"$cols\">&nbsp;</td></tr>\n";
                //restore old settings
            }
            $this->settings['where'] = $where . " and " . $this->get('id_query') . "!=" . $id;
            $this->settings['offset'] = $offset;
            $this->num_rows = $num_rows;
        }

        $res = pg_query($this->context->getDbConn(), $this->createQuery());
        $class = 'odd';
        for ($j = 0; $j < pg_num_rows($res); $j++) {
            $r = pg_fetch_row($res, $j);
            $out .= '<tr valign="top" class="list_row_' . $class . '" data-id="' . $r[$i_id] . '">';
            $class = ($class == 'odd' ? 'even' : 'odd');
            for ($i = 0; $i < count($this->fields); $i++) {
                $out .= "<td class=\"baycmslist-data-td\">$r[$i]</td>";
            }
            $out .= $this->getActions($r[$i_id], $r[$i_writeaccess] ?? 't');
            $out .= "</tr>\n";
        }
        $out .= "</table>";
        if ($GLOBALS['TE']->isBootstrap())
            $out .= '<div class="clearfix"></div>';
        $out .= $back_forward_html;
        if ($GLOBALS['TE']->isBootstrap())
            $out .= '<div class="clearfix"></div>';
        if ($this->settings['view'] ?? false)
            $out .= '
        <script>
        $("#' . $this->get('name') . ' td.baycmslist-data-td").on("click",function(){
            if(! $(this).parent().attr("data-id")) return;
            ' . (($_GET['js_select'] ?? false) ?
                    'var tr=$(this).parent()
                add(tr.attr("data-id"),tr.find("td:first-child").text())' :
                    'window.location.href="?' . $this->get('id_name') . '="+$(this).parent().attr("data-id")+"&' . $this->get('qs') . '";'
                ) . '
        });
        </script>';

        if ($_GET['js_select'] ?? false) {
            $out .= $this->t("Click on a row to select it", "Klicken Sie auf eine Zeile, um diese auszuwählen");
            if ($_GET['js_select'] == 1)
                $out .= " -- " . $GLOBALS['TE']->getActionLink(
                    "#",
                    $this->t('No selection', 'keine Auswahl'),
                    " onClick=\"add('','')\" ",
                    'remove'
                ) . "\n";
            $out .= "<br/>\n";
        }
        if ($this->settings['export'] ?? false) {
            $out .= "Export: ";
            $qs = $this->get('qs');
            if ($qs)
                $qs = "&$qs";

                $out .= $GLOBALS['TE']->getActionLink(
                    '?baycmsExportFormat=xlsx'.$qs,
                    'Download xlsx',
                    '',
                    'save'
                );
                $out .= " &nbsp; ";
                $out .= $GLOBALS['TE']->getActionLink(
                    '?baycmsExportFormat=csv'.$qs,
                    'Download csv',
                    '',
                    'save'
                );
            foreach (($this->settings['additional_export_formats'] ?? []) as $f) {
                $out .= " &nbsp; ";
                $out .= $GLOBALS['TE']->getActionLink($f['link'], $f['name'],$f['attr']??'',$f['icon']??'');
            }
        }
        return $out;

    }


}