<?php

namespace BayCMS\Fieldset;


class Domain extends Fieldset
{

    protected \BayCMS\Fieldset\Form $sform;
    protected \BayCMS\Fieldset\Form $eform;

    protected \BayCMS\Fieldset\BayCMSList $list;

    protected array $fields;
    protected ?int $id = null;

    public function __construct(\BayCMS\Base\BayCMSContext $context, array $settings)
    {
        if (!isset($settings['table']))
            throw new \BayCMS\Exception\missingData('You have to name a table');
        if (!isset($settings['name']))
            $settings['name'] = 'domain1';

        $this->context = $context;
        $this->settings = $settings;
    }


    static function autoCreate(\BayCMS\Base\BayCMSContext $context, array $settings)
    {
        $map = [
            'int8' => 'Number',
            'bit' => '',
            'varbit' => '',
            'bool' => 'Checkbox',
            'box' => '',
            'bytea' => '',
            'varchar' => 'TextInput',
            'bpchar' => 'TextInput',
            'cidr' => '',
            'circle' => '',
            'date' => 'Date',
            'float8' => 'Number',
            'inet' => 'TextInput',
            'int4' => 'Number',
            'interval' => '',
            'line' => '',
            'lseg' => '',
            'macaddr' => '',
            'money' => 'Number',
            'numeric' => 'Number',
            'path' => '',
            'point' => '',
            'polygon' => '',
            'float4' => 'Number',
            'int2' => 'Number',
            'text' => 'TextInput',
            'time' => 'Time',
            'timetz' => 'Time',
            'timestamp' => 'Datetime',
            'timestamptz' => 'Datetime'
        ];

        $domain = new Domain($context, $settings);
        if (!isset($settings['class_map']))
            $settings['class_map'] = null;
        if (!isset($settings['lfield_map']))
            $settings['lfield_map'] = null;
        if (!isset($settings['sfield_map']))
            $settings['sfield_map'] = null;

        $res = pg_query($context->getDbConn(), "select * from " . $domain->get('table') . " limit 1");

        $count = 0;
        for ($i = 0; $i < pg_num_fields($res); $i++) {
            $name = pg_field_name($res, $i);
            if ($name == 'id')
                continue;

            if (is_array($settings['lfield_map']))
                $lfield = $settings['lfield_map'][$name] ?? false;
            else
                $lfield = $count < ($settings['lfield_count'] ?? 3);

            if (is_array($settings['sfield_map']))
                $sfield = $settings['sfield_map'][$name] ?? false;
            else
                $sfield = $count <= ($settings['sfield_count'] ?? 1);

            $class = false;
            if (is_array($settings['class_map']))
                $class = $settings['class_map'][$name] ?? false;
            if (!$class) {
                $class = $map[pg_field_type($res, $i)] ?? false;
                if ($class)
                    $class = "\\BayCMS\\Field\\$class";
            }
            if ($settings['force_class'] ?? false)
                $class = $settings['force_class'];
            if (!$class)
                continue;
            $count++;
            $domain->addField(new $class($context, [
                'name' => ucfirst($name),
                'lfield' => $lfield,
                'sfield' => $sfield
            ]));

        }
        return $domain;
    }

    protected function createSearchForm()
    {
        $this->sform = new \BayCMS\Fieldset\Form(
            $this->context,
            ['name' => 'sform']
        );
        foreach ($this->fields as $f) {
            if (!$f->get('sfield'))
                continue;
            $this->sform->addField($f);
        }
        if ($this->settings['qs'] ?? false) {
            foreach (explode('&', $this->settings['qs']) as $f) {
                [$name, $value] = explode("=", $f);
                $this->sform->addField(new \BayCMS\Field\Hidden($this->context, [
                    'name' => $name,
                    'default_value' => urldecode($value)
                ]));
            }
        }
    }


    protected function createEditForm($upload = false)
    {
        $this->eform = new \BayCMS\Fieldset\Form(
            $this->context,
            [
                'name' => 'eform',
                'table' => $this->settings['table'],
                'uname' => $this->settings['uname'] ?? '',
                'id_name' => $this->settings['id_name'] ?? 'id',
                'qs' => $this->settings['qs'] ?? '',
                'delete_link' => $this->settings['delete_link'] ?? true,
                'object_link' => $this->settings['object_link'] ?? true
            ]
        );
        foreach ($this->fields as $f) {
            if ($f->get('no_efield'))
                continue;
            if ($upload && strstr(get_class($f), 'BayCMS\\Field\\Bilang')) {
                $f_nr = 0;
                foreach ($f->getFields() as $f) {
                    $this->eform->addField($f);
                    $f_nr++;
                    if ($f_nr == 2)
                        $f->set(['non_empty' => 0]);
                }
            } else
                $this->eform->addField($f);

        }
    }

    protected function createList($target = 'html')
    {
        $uname = $this->settings['uname'] ?? false;

        $settings = [
            'from' => $this->settings['from'] ?? $this->settings['table'] . " t" .
                ($uname ? ", objekt_verwaltung" . $this->context->get('row1', 'id') . " o" : ''),
            'where' => ($this->settings['where'] ?? ($uname ? "t.id=o.id" : 'true')) .
                ($this->settings['squery'] ?? ''),
            'qs' => $this->settings['qs'] ?? '',
            'order_by' => $this->settings['order_by'] ?? [1],
            'actions' => $this->settings['actions'] ?? ['edit_copy'],
            'action_sep' => $this->settings['action_sep'] ?? '</td><td>',
            'id_query' => 't.id',
            'offset' => $_GET['table1offset'] ?? 0,
            'id_name' => $this->settings['id_name'] ?? 'id',
            'step' => $this->settings['step'] ?? 20,
            'action_new' => $this->settings['action_new'] ?? true,
            'action_upload' => $this->settings['action_upload'] ?? false,
            'export' => $this->settings['export'] ?? !($_GET['js_select'] ?? false),
            'additional_export_formats' => $this->settings['additional_export_formats'] ?? [],
            'view' => true
        ];
        if ($target == 'html') {
            $settings['write_access_query'] = $this->settings['write_access_query'] ??
                ($uname ? 'check_objekt(t.id,' . $this->context->get('row1', 'id_benutzer') . ')' : 'true');

        }
        if ($target == 'export')
            $settings['step'] = -1;

        $this->list = new \BayCMS\Fieldset\BayCMSList(
            $this->context,
            $settings
        );
        foreach ($this->fields as $f) {
            if ($f->get('no_efield'))
                continue;
            if ($target != 'export' && !$f->get('lfield'))
                continue;
            if ($target == 'json') {
                $f->set(['description' => 'label']);
                $this->list->addField($f);
                $f2 = clone $f;
                $f2->set(['description' => 'value']);
                $this->list->addField($f2);
                break;
            }
            if ($f->get('no_add_to_query') && !$f->get('lfield'))
                continue;
            if ($target != 'html') {
                if (in_array(get_class($f), ['BayCMS\\Field\\Time', 'BayCMS\\Field\\Date', 'BayCMS\\Field\\Datetime', 'BayCMS\\Field\\Checkbox']))
                    $f = new \BayCMS\Field\TextInput($this->context, $f->get());

            }
            if ($target != 'html' && strstr(get_class($f), "BayCMS\\Field\\Bilang")) {
                foreach ($f->getFields() as $f) {
                    $this->list->addField($f);
                }
            } else
                $this->list->addField($f);
        }
        if ($target == 'json' && !($this->settings['squery'] ?? '')) {
            $where = $this->list->get('where');
            $f = $this->list->getField(0);
            $where .= " and " . $f->getSQL('t.') . " ilike '%" . pg_escape_string($this->context->getDbConn(), $_GET['json_query']) . "%'";
            $this->list->set(['where' => $where]);
        }

        if ($target == 'html' && isset($_GET['js_select'])) {
            $f = $this->list->getField(0);
            $sql = $f->getSQL('t.');
            $sql .= "||' '||js_add(t.id,$sql)||'<i>" . $this->t('take over', "übernehmen") . "</i></a>'";
            //            $f->set(['sql' => $sql]);
        }
    }

    protected function getHead()
    {
        $head = ($_GET['js_select'] ?? false ?
            ($this->de() ?
                $this->settings['name'] . ' auswählen' : 'Select ' . $this->settings['name']) :
            ($this->de() ?
                $this->settings['name'] . ' verwalten' : 'Manage ' . $this->settings['name']));

        $out = "<h3>$head</h3>\n";
        if ($_GET['js_select'] ?? false) {
            if (!($this->settings['qs'] ?? false))
                $this->settings['qs'] = 'js_select=' . $_GET['js_select'];
            else
                $this->settings['qs'] .= '&js_select=' . $_GET['js_select'];
            $out .= "<script>\n";
            switch ($_GET['js_select']) {
                case 'tiny':
                    $out .= "function add(id,name){
                       top.tinymce.activeEditor.windowManager.getParams().oninsert('" . $this->settings['tinyurl'] . "'+id,{'text':name});
                       top.tinymce.activeEditor.windowManager.close();
                       return false;
                    }";
                    break;
                case "n":
                    $out .= "function add(id, name)
                    \n{
                    top.sel[top.sel.length] = id;
                    top.selr[top.selr.length] = name;
                    top.frames[1].location = top.frames[1].location;
                    top.frames[1].location.reload();
                    return false;
            }";
                    break;
                case "1":
                    $out .= "function add(id, name)
                    \n{
                    opener.$_GET[target].value= id;
                    opener.$_GET[target]_dp.value= name;
                    window.close();
                    return false;
            }";
                    $this->settings['qs'] .= "&target=$_GET[target]";

                    break;
            }
            $out .= "\n</script>\n";
        }
        $out .= $this->settings['comment'] ?? '';
        return ['html' => $out];

    }

    public function getUpload()
    {
        if (($_GET['aktion'] ?? '') == 'upload' && !($_POST['delim'] ?? false)) {
            $form = new \BayCMS\Fieldset\Form($this->context, [
                'form_options' => ' enctype="multipart/form-data"',
                'submit' => 'Upload',
                'action' => "?aktion=upload"
            ]);
            $form->addField(new \BayCMS\Field\File($this->context, [
                'name' => 'uploaded',
                'description' => 'File (CSV, XLS, XLSX)',
                'help' => 'Bei Excel-Dateien wird nur das aktive Tabellenblatt verarbeitet.'
            ]));
            $form->addField(new \BayCMS\Field\Select($this->context, [
                'name' => 'delim',
                'description' => 'Spaltentrennzeichen (nur für CSV)',
                'values' => array(',', ';', 'TAB')
            ]));
            $form->addField(new \BayCMS\Field\Comment($this->context, [
                'name' => 'comment',
                'description' => 'Alternativ zum Datei Upload können Sie per Copy/Paste den Inhalt der Excel-Datei in das folgende Text-Feld kopieren'
            ]));
            $form->addField(new \BayCMS\Field\Textarea($this->context, [
                'name' => 'uploaded_txt',
                'description' => 'Excel-Datei Inhalt'
            ]));
            return ['finished' => true, 'html' => $form->getForm('Upload Data')];
        }

        $this->settings['delete_link'] = false;
        $this->settings['object_link'] = false;

        if (isset($_POST['delim'])) { //call from Form
            unset($_SESSION['domain_upload']);
            unset($_SESSION['domain_upload_resume']);
            $_SESSION['domain_upload_report'] = [];
            $fields = false;
            if ($_POST['uploaded_txt'] ?? false) {
                $content = $_POST['uploaded_txt'];
                $delim = "\t";
            } elseif (strstr(strtolower($_FILES['uploaded']['name']), '.xls')) {
                require_once("PHPExcel.php");
                $objPHPExcel = \PHPExcel_IOFactory::load($_FILES['uploaded']['tmp_name']);
                $objWorksheet = $objPHPExcel->getActiveSheet();
                $fields = [];
                $r = 0;

                foreach ($objWorksheet->getRowIterator() as $row) {
                    $cellIterator = $row->getCellIterator();
                    $cellIterator->setIterateOnlyExistingCells(FALSE); // This loops through all cells,
                    $has_data = false;
                    $col = [];
                    foreach ($cellIterator as $cell) {
                        $v = $cell->getValue();
                        if (\PHPExcel_Shared_Date::isDateTime($cell)) {
                            $v = date("Y-m-d H:i:s", \PHPExcel_Shared_Date::ExcelToPHP($v));
                        }
                        $col[] = $v;
                        if (!$has_data)
                            $has_data = $v > '';
                    }
                    if ($has_data)
                        $fields[] = $col;
                }
            } elseif (strstr(strtolower($_FILES['uploaded']['name']), '.csv')) {
                $fp = fopen($_FILES['uploaded']['tmp_name'], 'r');
                $content = '';
                while (!feof($fp)) {
                    $content .= fread($fp, 8192);
                }
                $content = mb_convert_encoding($content, 'UTF-8', "UTF-8, ISO-8859-1");
                $delim = $_POST['delim'];
                if ($delim == 'TAB')
                    $delim = "\t";

            } elseif (isset($_FILES['uploaded']['name'])) {
                throw new \BayCMS\Exception\invalidData("Unsupported file type " . $_FILES['uploaded']['name']);
            }

            if ($fields === false) {
                $fields = [];
                foreach (preg_split("/\r\n|\n|\r/", $content) as $l) {
                    if (!$l)
                        continue;
                    $fields[] = str_getcsv(trim($l), $delim);
                }
            }

        } elseif (isset($_SESSION['domain_upload'])) {
            $fields = $_SESSION['domain_upload'];
        }


        $field_names = [];
        $field_map = [];
        $id_column = -1;
        if (!isset($fields[0])) {
            throw new \BayCMS\Exception\invalidData("file is empty");
        }
        $this->createEditForm(true);
        for ($i = 0; $i < count($fields[0]); $i++) {
            if ($fields[0][$i] == 'id') {
                $id_column = $i;
                $field_names[] = 'id';
                continue;
            }
            $index = $this->eform->getFieldIndex($fields[0][$i]);
            if ($index == -1) {
                throw new \BayCMS\Exception\invalidData("Column " . $fields[0][$i] . " not found");
            }
            $field_names[] = $this->eform->getField($index)->getName();
            $field_map[$i] = $index;
        }

        $key_query = '';
        $key_count = 1;
        $key_names = [];
        foreach ($this->fields as $f) {
            if (!$f->get('key_field'))
                continue;
            if (!in_array($f->getName(), $field_names))
                throw new \BayCMS\Exception\missingData('Required column ' . $f->getName() . ' is missing.');
            $key_query .= ' and t.' . $f->getName() . '=$' . $key_count;
            $key_count++;
            $key_names[] = $f->getName();
        }


        if ($key_query) {
            pg_prepare($this->context->getRwDbConn(), 'key_query', 'select t.id from ' .
                ($this->get('uname') ? "objekt_verwaltung" . $GLOBALS['row1']['id'] . " o, " : '') . $this->get('table') . ' t 
                where true' . $key_query .
                ($this->get('uname') ? ' and o.id=t.id' : ''));
        }

        $i = 0;
        while ($f = $this->eform->getField($i)) {
            if (!in_array($f->getName(), $field_names))
                $this->eform->delField($i);
            $i++;
        }
        $this->eform->addField(new \BayCMS\Field\Hidden($this->context, [
            'name' => 'domain_upload_resume',
            'value' => 1,
            'no_add_to_query' => 1
        ]));

        if (isset($_POST['domain_upload_resume']) && isset($_SESSION['domain_upload_resume'])) {
            if ($_GET[$this->eform->get('id_name')] ?? false) {
                $this->eform->setId($_GET[$this->eform->get('id_name')]);
            }
            if ($this->eform->setValues($_POST)) {
                $_SESSION['domain_upload_report']['rows'][$_POST['domain_upload_resume']] = 'skipped';
                return ['finished' => false, 'html' => $this->eform->getForm("Upload Row " . $_POST['domain_upload_resume'])];
            }
            $_SESSION['domain_upload_report']['rows'][$_POST['domain_upload_resume']] = [$_GET[$this->eform->get('id_name')] ?? null, $this->eform->save()];
        }

        $upload_sql_prepared = array();
        $r_start = (isset($_SESSION['domain_upload_resume']) ? $_SESSION['domain_upload_resume'] : 1);
        unset($_SESSION['domain_upload']);
        unset($_SESSION['domain_upload_resume']);
        for ($r = $r_start; $r < count($fields); $r++) {
            while (count($field_names) > count($fields[$r])) {
                $fields[$r][] = null;
            }

            $key_array = array();
            foreach ($key_names as $n) {
                $key_array[] = $fields[$r][array_search($n, $field_names)];
            }

            $id = null;
            for ($c = 0; $c < count($fields[$r]); $c++) {
                if ($c == $id_column) {
                    $id = $fields[$r][$c];
                    continue;
                }

                if ($this->eform->getField($field_map[$c])->get('upload_sql')) {
                    if (!isset($upload_sql_prepared[$c])) {
                        pg_prepare($this->context->getRwDbConn(), 'upload_sql' . $c, );
                        $upload_sql_prepared[$c] = true;
                    }
                    if ($fields[$r][$c]) {
                        $res = pg_execute($this->context->getRwDbConn(), 'upload_sql' . $c, [$fields[$r][$c]]);
                        if (pg_num_rows($res))
                            list($fields[$r][$c]) = pg_fetch_row($res);
                        else {
                            $_SESSION['domain_upload_report']['foreign_keys'][$field_names[$c]][$r] = 1;
                            $fields[$r][$c] = '';
                        }
                    }
                }
            }

            if ($key_query && !$id) {
                $res = pg_execute($this->context->getRwDbConn(), 'key_query', $key_array);
                if (pg_num_rows($res) > 1) {
                    $_SESSION['domain_upload_report']['keys'][implode(", ", $key_array)] = 'dublicate';
                    continue;
                }
                if (pg_num_rows($res) == 1)
                    list($id) = pg_fetch_row($res, 0);
                else {
                    $id = null;
                    $_SESSION['domain_upload_report']['keys'][implode(", ", $key_array)] = 'not found';
                }

            }
            if (!$id && $this->get('no_create_on_upload')) {
                $_SESSION['domain_upload_report']['rows'][$r] = 'skipped';
                continue;
            }
            $this->id = $id;
            $this->eform->setId($id);
            $v = array_combine($field_names, $fields[$r]);
            $v['domain_upload_resume'] = $r;
            if ($this->eform->setValues($v)) {
                $_SESSION['domain_upload'] = $fields;
                $_SESSION['domain_upload_resume'] = $r + 1;
                $_SESSION['domain_upload_report']['rows'][$r] = 'skipped';

                return ['finished' => false, 'html' => $this->eform->getForm("Upload Row " . $r)];
            } else {
                $this->id = $this->eform->save();
                $_SESSION['domain_upload_report']['rows'][$r] = [$id, $this->id];

            }
        }

        if (isset($_SESSION['domain_upload_report'])) {
            $out = '<h2>Upload Report</h2>';
            foreach ($_SESSION['domain_upload_report']['rows'] as $r => $v) {
                $out .= "<b>$r</b>: ";
                if (is_array($v)) {
                    $out .= "<span style=\"color:#00bb00;\">";
                    if ($v[0] != $v[1])
                        $out .= " created with id=" . $v[1];
                    else
                        $out .= " updated id=" . $v[1];
                } else {
                    $out .= "<span style=\"color:#aa0000;\">" . $v;
                }
                $out .= "</span><br/>\n";
            }


            if (isset($_SESSION['domain_upload_report']['keys'])) {
                $out .= "
    <h3>Übersprungen:</h3><table>";
                foreach ($_SESSION['domain_upload_report']['keys'] as $key => $value) {
                    $out .= "<tr><td>$key</td><td>$value</td></tr>";
                }
                $out .= "</table>";
            }
            if (isset($_SESSION['domain_upload_report']['foreign_keys'])) {
                $out .= "<h3>Nicht gefundene Fremdschlüssel:</h3><table>";
                foreach ($_SESSION['domain_upload_report']['foreign_keys'] as $key => $value) {
                    $out .= "<tr><td>$key</td><td>" . implode("; ", array_keys($value)) . "</td></tr>";
                }
                $out .= "</table>";
            }

            unset($_SESSION['domain_upload_report']);
            unset($_SESSION['domain_upload']);
            unset($_SESSION['domain_upload_resume']);
            unset($_GET['aktion']);
            $_GET['id'] = $this->id;
            return ['finished' => true, 'html' => $out];
        }


    }

    public function getEdit()
    {
        $action = $_GET['aktion'] ?? '';
        $this->id = $_GET[$this->settings['id_name'] ?? 'id'] ?? null;
        $ok = 1;
        $error = '';
        if ($action == 'del') {
            if ($this->settings['uname'] ?? '') {
                $obj = new \BayCMS\Base\BayCMSObject($this->context);
                try {
                    $obj->load($this->id);
                    $obj->delete();
                } catch (\Exception $e) {
                    $ok = 0;
                    $error = $e->getMessage();
                }
            } else {
                try {
                    pg_query_params(
                        $this->context->getRwDbConn(),
                        'delete from ' . $this->settings['table'] . ' where id=$1',
                        [$this->id]
                    );
                } catch (\Exception $e) {
                    $ok = 0;
                    $error = $e->getMessage();
                }

            }
            if ($ok) {
                $message = $this->t('Deleted entry', 'Eintrag gelöscht');
                $this->id = null;
            } else
                $message = '';
            return ['html' => '', 'error' => $error, 'message' => $message, 'edit' => 0];
        }

        $error = '';
        $message = '';
        $html = '';
        $this->createEditForm();

        if ($this->id && !$action) {
            $this->eform->load($this->id);
            $html .= $this->eform->getTable();
        }

        if ($this->id && ($this->settings['copy'] ?? true)) {
            $this->eform->addField(new \BayCMS\Field\Checkbox(
                $this->context,
                [
                    'name' => 'save_as_copy',
                    'no_add_to_query' => 1,
                    'not_in_table' => 1,
                    'description' => $this->t('Save as new entry', 'Als neuen Eintrag speichern')
                ]
            ));
        }
        if ($action == 'save' && ($_POST['save_as_copy'] ?? false)) {
            $this->id = null;
        }
        if ($action == 'save') {
            if ($this->id)
                $this->eform->setId($this->id);
            $error = $this->eform->setValues($_POST);
            if ($error) {
                $action = 'edit';
                $error = $this->t('Please fill in all mandatory fields', 'Bitte füllen Sie das Formular vollständig aus');
            }
        }
        if ($action == 'save') {
            try {
                $this->id = $this->eform->save();
                $html .= $this->eform->getTable();
                $message = $this->t('Saved entry', 'Eintrag gespeichert');
            } catch (\Exception $e) {
                $error = $e->getMessage();
                $action = 'edit';
            }
        }
        if ($action == 'edit' || $action == 'copy') {
            if ($this->id)
                $this->eform->load($this->id);
            if ($action == 'copy')
                $this->id = null;
            if ($this->id)
                $head = $this->t('Edit Entry', 'Eintrag bearbeiten');
            else
                $head = $this->t('New Entry', 'Neuer Eintrag');
            $html .= $this->eform->getForm($head);
        }
        return ['html' => $html, 'error' => $error, 'message' => $message, 'edit' => ($action == 'edit'),'id'=>$this->id];
    }

    public function getSearch()
    {
        $this->createSearchForm();
        $this->sform->setValues($_GET);
        $html = $this->sform->getSearchForm() . "<br/>\n";
        $qs = $this->sform->getQS();
        if (!($this->settings['qs'] ?? false))
            $this->settings['qs'] = $qs;
        else
            $this->settings['qs'] .= "&" . $qs;
        if (!isset($this->settings['squery']) && isset($_GET[$this->sform->getField(0)->getName()])) {
            $this->settings['squery'] = " and " . $this->sform->getField(0)->getSQL("t.") . " ilike '%" .
                pg_escape_string($this->context->getDbConn(), $_GET[$this->sform->getField(0)->getName()]) .
                "%'";
        }
        return ['html' => $html, 'error' => '', 'message' => ''];
    }

    public function getList()
    {
        $this->createList();
        $html = $this->list->getTable($this->id);
        return ['html' => $html, 'error' => '', 'message' => ''];
    }

    public function getJSON()
    {
        $this->createList('json');
        return $this->list->getJSON();
    }


    public function pageEdit(){
        $search = $this->getSearch(); 
        $edit = $this->getEdit();
        if ($edit['error'])
            $GLOBALS['TE']->printMessage($edit['error'], 'danger');
        if ($edit['message'])
            $GLOBALS['TE']->printMessage($edit['message']);
        echo $edit['html'];
        if ($edit['edit'] && ($this->settings['no_table_on_edit'] ?? true))
            return $this->context->printFooter();
        ;

        echo $search['html'];
        echo $this->getList()['html'];
    }

    public function page()
    {
        $search = $this->getSearch(); //sets squery for export!

        if (isset($_GET['baycmsExportFormat'])) {
            $this->createList('export');
            $this->list->getExport($_GET['baycmsExportFormat'], $this->settings['name']);
        }

        if (isset($_GET['json_query'])) {
            echo $this->getJSON();
            exit();
        }

        $this->context->printHeader();
        echo $this->getHead()['html'];
        if (($_SESSION['domain_upload_resume'] ?? false) || ($_GET['aktion'] ?? '') == 'upload') {
            try {
                $upload = $this->getUpload();
                echo $upload['html'];
                if (!$upload['finished'])
                    return $this->context->printFooter();
            } catch (\Exception $e) {
                $GLOBALS['TE']->printMessage($e->getMessage(), 'danger');
            }
        }


        $edit = $this->getEdit();
        if ($edit['error'])
            $GLOBALS['TE']->printMessage($edit['error'], 'danger');
        if ($edit['message'])
            $GLOBALS['TE']->printMessage($edit['message']);
        echo $edit['html'];
        if ($edit['edit'] && ($this->settings['no_table_on_edit'] ?? true))
            return $this->context->printFooter();
        ;

        echo $search['html'];
        echo $this->getList()['html'];
        $this->context->printFooter();
    }

}