<?php

namespace BayCMS\Fieldset;

class Form extends Fieldset
{

    protected ?int $id = null;
    protected string $name;


    public function __construct(\BayCMS\Base\BayCMSContext $context, array $settings = [])
    {
        $defaults = [
            "method" => 'post',
            'action' => '?aktion=save',
            'write_access_query' => '',
            'div_names' => []
        ];
        if (!isset($settings['id_name']))
            $settings['id_name'] = 'id';
        $this->settings = array_merge($defaults, $settings);
        $this->name = $settings['name'] ?? 'form1';
        $this->context = $context;
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function setId($id)
    {
        $this->id = $id;
    }

    public function getName(): string
    {
        return $this->name;
    }


    public function getTable($settings = [])
    {
        $out = '<table ' . $GLOBALS['TE']->getCSSClass('table') . ">\n";
        foreach ($this->fields as $field) {
            $out .= $field->getTableRow();
        }
        $actions = '';
        if (($settings['delete_link'] ?? true) && $this->writeAccess())
            $actions .= $GLOBALS['TE']->getActionLink(
                "?aktion=del&" . $this->get('id_name') . "=" . $this->id . "&" . $this->get('qs'),
                $this->t("delete", "löschen"),
                " onClick=\"return confirm('" . $this->t("Are you sure?", "Wirklich löschen?") . "')\"",
                'del'
            ) . " &nbsp; ";

        if ($settings['object_link'] ?? ($this->settings['uname'] ?? false))
            $actions .= $GLOBALS['TE']->getActionLink(
                "../gru/objekt_detail.php?id_obj=" . $this->id,
                $this->t("Object Properties", "Objekt Eigenschaften"),
                '',
                'properties'
            ) . " &nbsp; ";

        if ($settings['edit_link'] ?? true) {
            if ($this->writeAccess())
                $actions .= $GLOBALS['TE']->getActionLink(
                    "?aktion=edit&" . $this->get('id_name') . "=" . $this->id . "&" . $this->get('qs'),
                    $this->t("edit", "bearbeiten"),
                    "",
                    'edit'
                ) . " &nbsp; ";
            else
                $actions .= $GLOBALS['TE']->getActionLink(
                    "?aktion=copy&" . $this->get('id_name') . "=" . $this->id . "&" . $this->get('qs'),
                    $this->t("copy", "kopieren"),
                    "",
                    'edit'
                ) . " &nbsp; ";
        }

        if (($settings['takeover_link'] ?? true) && ($_GET['js_select'] ?? false))
            $actions .= $GLOBALS['TE']->getActionLink(
                "#",
                $this->t("take over", "übernehmen"),
                " onClick=\"add('" . $this->id . "','" . addslashes($this->getField(0)->getValue()) . "')\"",
                'ok'
            ) . " &nbsp; ";



        if ($actions)
            $out .= "<tr><td colspan=2>$actions</td></tr>\n";
        $out .= "</table>\n";
        return $out;
    }

    private function getHelpJS()
    {
        $out = "<script>
        var " . $this->name . "_changed = false;";

        if ($GLOBALS['TE']->isBootstrap())
            $out .= "
   \$(document).ready(function(){
      $('[data-toggle=\"popover\"]').popover();   
  });</script>";
        else
            $out .= "
  var helptext=null;
  function hideHelp()
  {
    if(helptext!=null)
    {
       helptext.parentNode.removeChild(helptext);
       helptext=null;
    }
  }

  function showHelp(titel,beschr,id)
  {
    hideHelp();
    helptext= document.createElement('div');
  
  helptext.innerHTML='<p style=\"width:100%; text-align:right;\"><a onClick=\"hideHelp();\">[X]</a></p><p style=\"text-align:left;\"><b>'+titel+'</b><br/>'+beschr+'</p>';
  var ref = document.getElementById(id);
   var Parents = ref.offsetParent;
   var posx = ref.offsetLeft+ref.offsetWidth+15;
   var posy = ref.offsetTop;
   while (Parents) {
     posx+=Parents.offsetLeft;
     posy+=Parents.offsetTop;
    Parents = Parents.offsetParent;
    }
  helptext.style.position = 'absolute';
  helptext.style.left = posx + 'px';
  helptext.style.top = (posy-10) + 'px';
  helptext.style.width = '300px';
  helptext.style.backgroundColor='#ffffff';
  helptext.style.padding='10px';
  document.body.appendChild(helptext);
  }
  </script>";
        return $out;
    }

    public function getForm($heading = '')
    {

        $out = $this->getHelpJS();
        $action = $this->settings['action'];
        if ($this->id)
            $action .= "&" . ($this->settings['id_name'] ?? 'id_obj') . '=' . $this->id;
        if ($this->settings['qs'] ?? false)
            $action .= '&' . $this->settings['qs'];



        $out .= '<form method="' . $this->settings['method'] .
            '" id="' . $this->name .
            '" name="' . $this->name .
            '" action="' . $action . '" enctype="multipart/form-data"
             accept-charset="UTF-8" ' . ($this->settings['form_options'] ?? '') . '>' . "\n";
        $footnotes = [];
        $div_id = '';
        $first_div = true;
        $out .= "<fieldset>\n<legend>$heading</legend>\n";
        foreach ($this->fields as $field) {
            if ($div_id != $field->get('div_id')) {
                if ($first_div)
                    $out .= "<!-- INSERT_FIELDSET_TAB -->";
                else
                    $out .= "</div>";
                $div_id = $field->get('div_id');
                if (!isset($this->settings['div_names'][$div_id])) {
                    $this->settings['div_names'][$this->nameToId($div_id)] = $div_id;
                }
                $out .= '
                <div class="formclass_fieldset" id="' . $this->nameToId($div_id) . '" ' . (!$first_div ? 'style="display:none;"' : '') . '>';
                $first_div = false;
            }
            $out .= $field->getFormRow($this);
            $out .= $field->get('post_input');
            $fn = $field->getFootnote();
            if ($fn)
                $footnotes[$fn[0]] = $fn[1];
        }
        if ($div_id)
            $out .= "</div>";
        $out .= '</fieldset>';
        $out = str_replace('<!-- INSERT_FIELDSET_TAB -->', $this->getFieldsetTab(), $out);

        //Buttons
        if ($GLOBALS['TE']->isBootstrap())
            $class = "btn btn-primary";
        else
            $class = "button";
        $out .= '
        <fieldset><div class="formrow">
        <input class="' . $class . '" type="reset" value="' . $this->t("reset", "zurücksetzen") . '"        
         onClick="' . $this->name . '_changed==false;">' . "\n";
        if ($this->settings['cancel_button'] ?? true)
            $out .= '<input class="' . $class . '" type="button" value="' . $this->t("cancel", "abbrechen") . '"
        onClick="if(' . $this->name . '_changed==false || confirm(\'' . $this->t("Are you sure?", "Wirklich abbrechen?") . '\')){ 
          location.href=\'?' . ($this->settings['qs'] ?? '') . '\';
            } else { return; }">' . "\n";

        if ($this->id !== null && ($this->settings['delete_link'] ?? false) && $this->writeAccess())
            $out .= '<input class="' . $class . '" type="button" onClick="if(confirm(\'' . $this->t("Are you sure?", "Wirklich löschen?") . '\')){
            location.href=\'?aktion=del&' . $this->settings['id_name'] . '=' . $this->id . '\';
          } else {
            return;
          }" value="' . $this->t("delete", 'löschen') . '">' . "\n";

        if (
            $this->id !== null && ($this->settings['object_link'] ??
                ($this->settings['uname'] ?? false))
        )
            $out .= '<input type="button" class="' . $class . '" onClick="location.href=\'../gru/objekt_detail.php?id_obj=' . $this->id . '\'" 
          value="' . $this->t('Object Properties', "Objekt Eigenschaften") . '"> ' . "\n";

        $out .= '<input class="' . $class . '" type="submit" value="' .
            ($this->settings['submit'] ?? $this->tr('save', 'speichern')) . '">
        </div></fieldset>' . "\n";

        $out .= '</form>' . "\n";
        $out .= '<script>
        $(\'form#' . $this->name . ' :input\').change(function(){
           ' . $this->name . '_changed=true; 
      });
      $(\'form#' . $this->name . '\').submit(function(e) {
        $(\':disabled\').each(function(e) {
            $(this).removeAttr(\'disabled\');
        })
      });
    
        </script>';
        //Footnotes
        foreach ($footnotes as $key => &$value) {
            $out .= $key . " " . $this->tr($value) . "<br/>\n";
        }
        $out .= $this->settings['postform_html'] ?? '';
        return $out;
    }

    private function tr(string $string)
    {
        if ($this->context->get('lang') == 'de')
            return $string;
        return Form::$tr[$string] ?? $string;
    }

    private function getFieldsetTab()
    {
        $out = '';
        if (!count($this->settings['div_names']))
            return $out;
        if ($GLOBALS['TE']->isBootstrap()) {
            $out .= '<ul class="nav nav-tabs">
          ';
            $active = true;
            foreach ($this->settings['div_names'] as $id => $name) {
                $out .= '<li class="' . ($active ? 'active' : '') . '">
            <a href="#" class="formclass_tablink" data-id="' . $id . '" id="'.$id.'_tab">' . $name . '</a></li>' . "\n";
                $active = false;
            }
            $out .= '</ul>';
            $out .= '<script>
        $(".formclass_tablink").on("click",function(){
            $(".formclass_tablink").parent().removeClass("active")
            $(this).parent().addClass("active")
            $(".formclass_fieldset").css("display","none")
            var id=$(this).attr("data-id")
            $("#"+id).css("display","block")
        })
        </script>';
        } else {
            $rows=[];
            $row_length=0;
            $row_index=0;
            $max_length=($GLOBALS['TEMPLATE_REITER_MAX_LENGTH']??89);
            foreach ($this->settings['div_names'] as $id => $name) {
                $rows[$row_index][$id]=$name;
                $row_length+=4+mb_strlen($name,'UTF-8');
                if($row_length>$max_length){
                    $row_index++;
                    $row_length=0;
                }
            }

            krsort($rows);
            $count=0;
            $out='<div id="navsite"><ul id="'.$this->name.'_tab">';
            $active=true;
            foreach($rows as $k=>$r){
                foreach($r as $id=>$name){
                    $out.='<li class="'.$this->name.'_tabrow'.$k.'"><a href="#" class="formclass_tablink '.(!$k && $active?'active':'').'" 
                    data-id="'.$id.'" id="'.$id.'_tab">'.$name.'</a></li>';
                    if(! $k && $active) $active=false;
                }
                $out.='<br class="'.$this->name.'_tabrow'.$k.'"/>';
            }
            $out.='</ul></div>';
            $out .= '<script>
        $(".formclass_tablink").on("click",function(){
            $(".formclass_tablink").removeClass("active")
            $(this).addClass("active")
            $(".formclass_fieldset").css("display","none")
            var id=$(this).attr("data-id")
            $("#"+id).css("display","block")
            /* move rows... */
            var class_name=$(this).parent().attr("class")
            var ul=$(this).parent().parent()
            console.log(class_name)
            
            $("."+class_name).each(function(){
                $(this).appendTo("#'.$this->name.'_tab")
            })
        })
        </script>';
        }

        return $out;

    }

    public function save()
    {
        $table = $this->settings['table'] ?? false;
        if (!$table) {
            throw new \BayCMS\Exception\missingData("Cannot save form without setting table");
        }

        pg_query($this->context->getRwDbConn(), 'begin');
        $uname = $this->settings['uname'] ?? false;
        if ($uname) {
            $obj = new \BayCMS\Base\BayCMSObject($this->context);
            $de = $this->settings['de'] ?? ($_POST['de'] ?? '');
            $en = $this->settings['en'] ?? ($_POST['en'] ?? '');
            $id_parent = $this->settings['id_parent'] ?? ($_POST['id_parent'] ?? null);
            $stichwort = $this->settings['stichwort'] ?? ($_POST['stichwort'] ?? '');
            if (!is_null($this->id))
                $obj->load($this->id);
            $obj->set(['de' => $de, 'en' => $en, 'id_parent' => $id_parent, 'stichwort' => $stichwort, 'uname' => $uname, 'id_art' => null]);
            $id = $obj->save(false);

        } else {
            //Table only
            if (is_null($this->id)) {
                $res = pg_query($this->context->getRwDbConn(), "select nextval('" . $table . "_id_seq')");
                [$id] = pg_fetch_row($res, 0);
            }

        }
        $i = 1;
        $insert_query_fields = '';
        $insert_query_values = '';
        $update_query = 'update ' . $table . ' set ';
        $values = [];
        foreach ($this->getValues() as $f => $v) {
            $comma = $i > 1 ? ', ' : '';
            $insert_query_fields .= $comma . $f;
            $insert_query_values .= $comma . '$' . $i;
            $update_query .= $comma . $f . '=$' . $i;
            $i++;
            $values[] = $v;
        }

        if (is_null($this->id)) {
            $query = 'insert into ' . $table . '(' . $insert_query_fields . ',id) 
                values (' . $insert_query_values . ',$' . $i . ')';
            $this->id = $id;
        } else {
            $query = $update_query . ' where id=$' . $i;
        }
        $values[] = $this->id;

        $ok = pg_query_params($this->context->getRwDbConn(), $query, $values);
        foreach ($this->fields as $f) {
            $ok = $f->save($this);
            if (!$ok) {
                break;
            }
        }

        if ($ok) {
            pg_query($this->context->getRwDbConn(), 'commit');
            return $this->id;
        }
        return false;
    }

    public function load(int $id)
    {
        $table = $this->settings['table'] ?? false;
        if (!$table) {
            throw new \BayCMS\Exception\missingData("Cannot load form without setting table");
        }

        $res = pg_query_params($this->context->getRwDbConn(), 'select * from ' . $table . ' where id=$1', [$id]);
        if (!pg_num_rows($res)) {
            throw new \BayCMS\Exception\notFound('Could not load row with id=' . $id);
        }
        $r = pg_fetch_array($res, 0);
        $this->id = $id;
        $this->setValues($r);
    }

    public function writeAccess()
    {
        if (isset($this->settings['write_access']))
            return $this->settings['write_access'];
        if (($this->settings['uname'] ?? false) && !($this->settings['write_access_query'] || '')) {
            $this->settings['write_access_query'] = 'check_objekt($1,' .
                $this->context->get('row1','id_benutzer',0) . ')';
        }

        if ($this->settings['write_access_query']) {
            $res = pg_query_params(
                $this->context->getRwDbConn(),
                'select ' . $this->settings['write_access_query'] . ' from ' .
                $this->settings['table'] . ' where id=$1',
                [$this->id]
            );
            if (!pg_num_rows($res))
                return false;
            $r = pg_fetch_row($res, 0);
            $this->settings['write_access'] = ($r[0] == 't' ? true : false);
        } else
            $this->settings['write_access'] = true;

        return $this->settings['write_access'];
    }


    public function setValues(array &$values): int
    {
        $error = 0;
        foreach ($this->fields as $field) {
            $error += $field->setValueFromArray($values, $this->id);
        }
        return $error;
    }

    public function getValues(): array
    {
        $values = [];
        foreach ($this->fields as $field) {
            if ($field->noAddToQuery())
                continue;
            $v = $field->getValue();
            if (is_array($v)) {
                foreach ($v as $k => $v) {
                    $values[$k] = $v;
                }
            } else
                $values[$field->getName()] = $v;

        }
        return $values;
    }

    public function getSearchForm($with_description = false)
    {
        $out = '<form id="' . $this->name . '" name="' . $this->name . '">';
        foreach ($this->fields as $f) {
            $out .= $f->getInput($this);
            if ($with_description)
                $out .= $f->getDescription(false);
        }
        $out .= '<input type="submit" value="' . $this->t('search', 'suchen') . '"></form>';
        return $out;
    }

    public function getQS()
    {
        $qs = [];
        foreach ($this->getValues() as $k => $v) {
            if ($v)
                $qs[] = $k . "=" . urlencode($v);
        }
        return implode('&', $qs);
    }

}