<?php

namespace BayCMS\Fieldset;
use BayCMS\Page\Document;

class QueryBuilder extends Fieldset
{

    private ?int $id = null;
    private \BayCMS\Fieldset\Form $form;
    private array $value;
    private \BayCMS\Fieldset\BayCMSList $list;

    public function __construct(\BayCMS\Base\BayCMSContext $context, array $settings)
    {
        $this->context = $context;
        if (!isset($settings['num_order']))
            $settings['num_order'] = 3;
        if (!isset($settings['num_expert']))
            $settings['num_expert'] = 3;
        if (!isset($settings['num_prefilter']))
            $settings['num_prefilter'] = 3;
        if (!isset($settings['num_filter']))
            $settings['num_filter'] = 3;
        if (!isset($settings['name']))
            $settings['name'] = 'qb';
        $this->settings = $settings;
    }


    public function load(int $id = null)
    {
        if (!is_null($id))
            $this->id = $id;
        if (is_null($this->id))
            return;

        $res = pg_query_params(
            $this->context->getRwDbConn(),
            'select * from qb2 where id=$1',
            [$this->id]
        );
        if (!pg_num_rows($res))
            throw new \BayCMS\Exception\notFound('No query for id=' . $this->id);

        $r = pg_fetch_array($res, 0);
        $v = json_decode($r['json'], true);

        $this->setValues($v);
        $this->value = $v;
    }

    public function getEdit(): mixed
    {
        $action = $_GET['aktion'] ?? false;
        if ($action == 'del' && ($_GET['id'] ?? false)) {
            $obj = new \BayCMS\Base\BayCMSObject($this->context);
            $obj->load($_GET['id']);
            $obj->delete();
            $action = false;
            unset($_GET['id']);
            return '';
        }
        if ($_GET['id'] ?? false) {
            $this->load($_GET['id']);
        }

        $this->createEditForm();
        if (($_GET['id'] ?? false) && $action=='edit') {
            $this->form->setId(($_GET['id']));
            $this->form->setValues($this->value);
        }
        $error = false;
        $post_form = '';
        if ($_GET['aktion'] == 'save') {
            $this->form->setValues($_POST);
            $v = $this->form->getValues();
            $error = $this->setValues($v);
        }
        if ($error) {
            $_GET['aktion'] = 'edit';
            $post_form = '<script>
            $("#filter_tab").trigger("click")
            </script>';
        }
        if ($action == 'save' && ($_POST['save_as_copy'] ?? false)) {
            $this->id = null;
            unset($_GET['id']);
        }
        if ($_GET['aktion'] == 'save') {
            $obj = new \BayCMS\Base\BayCMSObject($this->context);
            if ($this->id)
                $obj->load($this->id);
            $obj->set([
                'de' => $this->form->getField($this->get('name') . '_qname')->getValue(),
                'uname' => 'qb2'
            ]);
            pg_query($this->context->getRwDbConn(), 'begin');
            $this->id = $obj->save(false);
            pg_query_params(
                $this->context->getRwDbConn(),
                ($_GET['id'] ?? false ?
                    'update qb2 set name=$2, class=$3, min_power=$4, description=$5, json=$6 where id=$1' :
                    'insert into qb2(id,name,class,min_power,description,json) values($1,$2,$3,$4,$5,$6)'),
                [
                    $this->id,
                    $this->form->getField($this->get('name') . '_qname')->getValue(),
                    get_class($this),
                    $this->form->getField($this->get('name') . '_qvis')->getValue(),
                    $this->form->getField($this->get('name') . '_qdesc')->getValue(),
                    json_encode($this->form->getValues())
                ]
            );
            pg_query($this->context->getRwDbConn(), 'commit');
        }

        if ($_GET['aktion'] == 'edit') {
            $out = $this->form->getForm("Edit Query");
            $out .= $post_form;
            return ['html' => $out, 'error' => $error];
        }
        return false;
    }

    public function getFieldTable()
    {
        $out = '<table ' . $GLOBALS['TE']->getCSSClass('table') . '>';
        foreach ($this->fields as $f) {
            $out .= '<tr><td>' . $f->getDescription(false) . '</td><td>' . $f->getSQL("t.") . '</td></tr>' . "\n";
        }
        $out .= '</table>';
        return $out;
    }

    public function createEditForm(): void
    {
        $out = '';
        $this->form = new Form($this->context, [
            'table' => 'qb2',
            'uname' => 'qb2'
        ]);
        $this->form->addField(new \BayCMS\Field\TextInput(
            $this->context,
            [
                'name' => $this->get('name') . '_qname',
                'description' => 'Query Name',
                'non_empty' => 1
            ]
        ));
        $this->form->addField(new \BayCMS\Field\Select(
            $this->context,
            [
                'name' => $this->get('name') . '_qvis',
                'description' => $this->t('Visibility', 'Sichtbarkeit'),
                'db_query' => 'select power as value,
                 non_empty(' . $this->context->get('lang') . ',' . $this->context->get('lang2') . ') as description
                 from power where power<=1000 order by 1 desc',
                'non_empty' => 1
            ]
        ));
        $this->form->addField(new \BayCMS\Field\Textarea(
            $this->context,
            [
                'name' => $this->get('name') . '_qdesc',
                'description' => $this->t('Comment', 'Bemerkung')
            ]
        ));
        if ($_GET['id'] ?? false) {
            $this->form->addField(new \BayCMS\Field\Checkbox(
                $this->context,
                [
                    'name' => 'save_as_copy',
                    'no_add_to_query' => 1,
                    'not_in_table' => 1,
                    'description' => $this->t('Save as new entry', 'Als neuen Eintrag speichern')
                ]
            ));
        }

        $field_values = [];
        foreach ($this->fields as $f) {
            $field_values[] = [$f->getName(), $f->getDescription(false), $f->get('div_id')];
            $this->form->addField(new \BayCMS\Field\Checkbox($this->context, [
                'name' => $f->getName(),
                'description' => $f->getDescription(false),
                'default_value' => $f->get('lfield'),
                'div_id' => $f->get('div_id')
            ]));
        }
        $this->form->addField(new \BayCMS\Field\Comment($this->context, [
            'name' => 'expert_help',
            'description' => '<a href="#" onClick="window.open(\'?help\', \'Help\', \'toolbar=no,menubar=no,scrollbars=yes,width=700,height=500\');">Table with field SQL</a>',
            'div_id' => $this->t('Expert Fields', 'SQL Felder')
        ]));
        for ($i = 0; $i < ($this->settings['num_expert']); $i++) {
            $this->form->addField(new \BayCMS\FieldExtra\QBSQLField($this->context, [
                'name' => $this->get('name') . '_expert' . $i,
                'values' => $field_values,
                'div_id' => $this->t('Expert Fields', 'SQL Felder'),
                'nr' => $i
            ]));
        }

        if (is_array($this->get('prefilter'))) { //format [['name'=>'XXX','sql'=>''],...]
            $values = [];
            foreach ($this->settings['prefilter'] as $k => $v) {
                $values[] = [$k, $v['name']];
            }
            for ($i = 0; $i < ($this->settings['num_prefilter']); $i++) {
                $this->form->addField(new \BayCMS\FieldExtra\QBPreFilter(
                    $this->context,
                    [
                        'name' => $this->get('name') . '_prefilter' . $i,
                        'values' => $values,
                        'nr' => $i,
                        'div_id' => $this->t('Pre-Filter', 'Vorfilter')
                    ]
                ));
            }
        }


        for ($i = 0; $i < ($this->settings['num_filter']); $i++) {
            $this->form->addField(new \BayCMS\FieldExtra\QBFilter($this->context, [
                'name' => $this->get('name') . '_filter' . $i,
                'values' => $field_values,
                'div_id' => 'Filter',
                'nr' => $i
            ]));
        }
        for ($i = 0; $i < ($this->settings['num_order']); $i++) {
            $this->form->addField(new \BayCMS\FieldExtra\QBOrder($this->context, [
                'name' => $this->get('name') . '_order' . $i,
                'values' => $field_values,
                'div_id' => 'Order'
            ]));
        }
    }

    public function getReplacementTable()
    {
        $this->createList();
        $out = '<b>Für Datensatz Nr. 1 müssen die folgenden Feldnamen im PDF-Formular genutzt werden:</b><br/>
        <table ' . $GLOBALS['TE']->getCSSClass('table') . '>';

        foreach ($this->list->getFields() as $f) {
            $out .= '<tr><td>' . $f->getDescription(false) . '</td><td>f1_' . $f->getName() . '</td></tr>' . "\n";
        }
        $out .= "</table>";
        $out .= 'Feldnamen für Datensatz Nr 2 entsprechend f2_xxx ...<br/>
        <br/>
        Für docx und odt bitte Felder in der Form ${Feldname} verwenden.';
        return $out;
    }

    protected function createList($exclude=false)
    {
        $settings = [
            'from' => $this->settings['list_from'],
            'where' => $this->settings['list_where'],
            'order_by' => $this->settings['order_by'] ?? [1],
            'actions' => $this->settings['actions'] ?? [],
            'action_sep' => $this->settings['action_sep'] ?? '</td><td>',
            'id_query' => 't.id',
            'step' => $this->settings['step'] ?? -1,
            'view' => $this->settings['view'] ?? false
        ];
        if($exclude){
            $settings['where'].=" and t.id not in 
            (select t.id_row from qb2_rows t, qb2_run r, objekt o
            where t.id_run=r.id and r.id=o.id and o.id_obj=".
            pg_escape_string($this->context->getDbConn(),$exclude).")";
        }

        $this->list = new \BayCMS\Fieldset\BayCMSList(
            $this->context,
            $settings
        );
        foreach ($this->fields as $f) {
            if (!$f->get('lfield'))
                continue;

            $this->list->addField($f);
        }

        $i = 0;
        while (isset($this->value[$this->get('name') . '_expert' . $i . '_name'])) {
            if ($this->value[$this->get('name') . '_expert' . $i . '_sql']) {
                $this->list->addField(new \BayCMS\Field\TextInput($this->context, [
                    'name' => 'expert' . $i,
                    'description' => $this->value[$this->get('name') . '_expert' . $i . '_name'],
                    'sql' => $this->value[$this->get('name') . '_expert' . $i . '_sql']
                ]));
            }
            $i++;
        }


    }


    public function setValues(&$v): string|bool
    {
        $this->settings['list_from'] = $this->settings['from'] ?? $this->settings['table'] . " t" .
            ($this->get('uname') ? ", objekt_verwaltung" . $this->context->get('row1', 'id') . " o" : '');
        $this->settings['list_where'] = $this->settings['where'] ?? ($this->get('uname') ? "t.id=o.id" : 'true');


        //Selected fields
        foreach ($this->fields as $f) {
            if (isset($v[$f->getName()]) && $v[$f->getName()] != 'f' && $v[$f->getName()])
                $f->set(['lfield' => 1]);
            else
                $f->set(['lfield' => 0]);
        }

        //PreFilter
        $prefilter = '';
        try {
            $prefilter = $this->getPreFilter($v, 't.');
        } catch (\Exception $e) {
            return $e->getMessage();
        }
        if ($prefilter)
            $prefilter = " and ($prefilter)";
        $this->settings['list_from'] = str_replace('${prefilter}', $prefilter, $this->settings['list_from']);

        //Filter
        try {
            $squery = $this->getFilter($v, 't.');
        } catch (\Exception $e) {
            return $e->getMessage();
        }
        if ($squery)
            $squery = " and ($squery)";
        $this->settings['list_where'] .= $squery;

        //Order by
        $name = $this->get('name') . '_order';
        $i = 0;

        $this->settings['order_by'] = [];
        while (isset($v[$name . $i . '_name'])) {
            if (!$v[$name . $i . '_name'])
                break;
            $field_sql = $this->getField($v[$name . $i . '_name'])->getSQL('t.');
            $desc = $v[$name . $i . '_desc'] ?? false;
            if ($desc == 'f')
                $desc = false;
            $this->settings['order_by'][] = $field_sql . ($desc ? ' desc' : '');
            $i++;
        }
        if (!count($this->settings['order_by']))
            $this->settings['order_by'] = [1];
        if ($i >= $this->settings['num_order'])
            $this->settings['num_order'] = $i + 1;
        return false;
    }

    public function getPreFilter(&$v, $prefix = 't.')
    {
        $name = $this->get('name') . '_prefilter';
        $i = 0;
        $out = '';
        $brackets = 0;

        while (isset($v[$name . $i . '_name'])) {
            if (!$v[$name . $i . '_name'])
                break;
            if ($i && !$v[$name . $i . '_con'])
                throw new \BayCMS\Exception\invalidData('you have to set and/or in line ' . ($i + 1));
            $out .= ' ' . $v[$name . $i . '_con'] . ' ';
            if ($v[$name . $i . '_bo']) {
                $brackets++;
                $out .= "(";
            }
            $out .= $this->settings['prefilter'][$v[$name . $i . '_name']]['sql'];
            if ($v[$name . $i . '_bc']) {
                if (!$brackets)
                    throw new \BayCMS\Exception\invalidData('closing bracket but no opening bracket in line' . ($i + 1));
                $brackets--;
                $out .= ")";
            }
            $i++;
        }
        if ($brackets)
            throw new \BayCMS\Exception\invalidData('missing closing bracket');

        if ($i >= ($this->settings['num_prefilter']))
            $this->settings['num_prefilter'] = $i + 1;
        return $out;
    }


    public function getFilter(&$v, $prefix = 't.')
    {
        $name = $this->get('name') . '_filter';
        $i = 0;
        $out = '';
        $brackets = 0;

        while (isset($v[$name . $i . '_name'])) {
            if (!$v[$name . $i . '_name'])
                break;
            $field_sql = $this->getField($v[$name . $i . '_name'])->getSQL($prefix);
            if ($i && !$v[$name . $i . '_con'])
                throw new \BayCMS\Exception\invalidData('you have to set and/or in line ' . ($i + 1));
            $out .= ' ' . $v[$name . $i . '_con'] . ' ';
            if ($v[$name . $i . '_bo']) {
                $brackets++;
                $out .= "(";
            }
            if ($i && !$v[$name . $i . '_op'])
                throw new \BayCMS\Exception\invalidData('you have to set an operator in line' . ($i + 1));

            switch ($v[$name . $i . '_op']) {
                case 'is null':
                    $out .= $field_sql . ' is null';
                    break;
                case 'null/empty':
                    $out .= '(' . $field_sql . ' is null or ' . $field_sql . '=\'\')';
                    break;
                default:
                    $val = $v[$name . $i . '_val'];
                    if (!is_numeric($val))
                        $val = "'" . pg_escape_string($this->context->getDbConn(), $val) . "'";
                    $out .= $field_sql . ' ' . $v[$name . $i . '_op'] . ' ' . $val;
            }
            if ($v[$name . $i . '_bc']) {
                if (!$brackets)
                    throw new \BayCMS\Exception\invalidData('closing bracket but no opening bracket in line' . ($i + 1));

                $brackets--;
                $out .= ")";
            }
            $i++;
        }
        if ($brackets)
            throw new \BayCMS\Exception\invalidData('missing closing bracket');

        if ($i >= ($this->settings['num_filter']))
            $this->settings['num_filter'] = $i + 1;
        return $out;
    }

    public function getTabNavigation()
    {
        $names = ['Query', 'Documents', 'Emails'];
        $urls = [
            $_SERVER['SCRIPT_NAME'],
            $_SERVER['SCRIPT_NAME'] . '/Document',
            $_SERVER['SCRIPT_NAME'] . '/Email'
        ];

        $nav = new \BayCMS\Util\TabNavigation($this->context, [
            'names' => $names,
            'urls' => $urls
        ]);
        return $nav->getNavigation();
    }

    public function getActions()
    {
        $out = '';
        $out .= $GLOBALS['TE']->getActionLink(
            '?id=' . $this->id . '&aktion=edit',
            $this->t('edit', 'bearbeiten'),
            '',
            'edit'
        );
        $out .= ' ';
        $out .= $GLOBALS['TE']->getActionLink(
            '?id=' . $this->id . '&aktion=del',
            $this->t('delete', 'löschen'),
            ' onClick="return confirm(\'' . $this->t('Are you sure?', 'Wirklich löschen?') . '\')"',
            'del'
        );
        $out .= ' ';
        $out .= $GLOBALS['TE']->getActionLink(
            '?id=' . $this->id . '&baycmsExportFormat=xlsx',
            'Download xlsx',
            '',
            'save'
        );
        $out .= ' ';
        $out .= $GLOBALS['TE']->getActionLink(
            '?id=' . $this->id . '&baycmsExportFormat=csv',
            'Download csv',
            '',
            'save'
        );
        $out .= ' ';
        $out .= $GLOBALS['TE']->getActionLink(
            $_SERVER['SCRIPT_NAME'] . '/Document?id_query=' . $this->id . '&aktion=edit',
            $this->t('New Document', 'Neues Dokument'),
            '',
            'file'
        );
        $out .= ' ';
        
        if (isset($this->settings['email_fields'])) {
            $out .= $GLOBALS['TE']->getActionLink(
                $_SERVER['SCRIPT_NAME'] . '/Email?id_query=' . $this->id . '&aktion=edit',
                $this->t('New Email', 'Neues E-Mail'),
                '',
                'envelope'
            );
            $out .= ' ';
        }


        return $out;
    }

    public function getList(int|bool $exclude=false)
    {
        $this->createList($exclude);
        return $this->list;
    }

    public function page($settings=[])
    {
        if (isset($_GET['baycmsExportFormat']) && $_GET['id'] ?? false) {
            $this->load($_GET['id']);
            $this->createList();
            $this->list->getExport($_GET['baycmsExportFormat'], $this->settings['name']);
        }

        if (isset($_GET['help'])) {
            $this->context->set('no_frame', true);
            $this->context->set('frameable', true);
            $this->context->printHeader();
            echo $this->getFieldTable();
            $this->context->printFooter();
        }

        if (!isset($this->settings['email_fields'])) {
            if ($f = $this->getField('email')) {
                $this->settings['email_fields'] = [$f->getName()];
            }
        }

        $this->context->initTemplate();
        $head=$settings['pre_content']??'';
        $head .= $this->getTabNavigation();

        //Check PATH_INFO
        if ($_SERVER['PATH_INFO'] ?? false) {
            $path = explode('/', $_SERVER['PATH_INFO']);
            switch ($path[1]) {
                case 'Document':
                    $doc = new Document($this->context, [
                        'class' => get_class($this)
                    ]);
                    $doc->page(['pre_content' => $head]);
                    break;
                case 'Email':
                    if(! isset($this->settings['email_fields'])){
                        $this->context->printHeader();
                        echo $head;
                        $GLOBALS['TE']->printMessage('There is no email field in the table.');
                        $this->context->printFooter();
                    }
                    break;
            }

        }
        $this->context->printHeader();
        echo $head;

        if (isset($_GET['aktion'])) {
            $edit = $this->getEdit();
            if ($edit) {
                if ($edit['error'])
                    $GLOBALS['TE']->printMessage($edit['error'], 'danger');
                echo $edit['html'];
                $this->context->printFooter();
            }
        }

        if ($_GET['id'] ?? false)
            $this->id = $_GET['id'];

        if ($this->id) {
            $this->load();
            $this->createList();
            echo $this->getActions() . "<br/>";
            echo $this->list->getTable();
            $this->context->printFooter();
        }


        //List of Queries
        $list = new \BayCMS\Fieldset\BayCMSList($this->context, [
            'from' => 'qb2 t, objekt_verwaltung' . $this->context->get('row1', 'id') . ' o',
            'where' => 't.id=o.id and t.class=\'' . pg_escape_string($this->context->getDbConn(), get_class($this)) . '\'',
            'id_query' => 't.id',
            'actions' => ['edit_copy', 'del'],
            'action_sep' => ' ',
            'action_new' => 1,
            'view' => 1
        ]);
        $list->addField(new \BayCMS\Field\TextInput($this->context, [
            'name' => 'name',
            'description' => 'Query'
        ]));
        echo $list->getTable($this->id);
        $this->context->printFooter();

    }

}