<?php

namespace BayCMS\Page;
use BayCMS\Field\TextInput;

class Document extends Page
{

    private \BayCMS\Fieldset\QueryBuilder $qb;
    private \BayCMS\Fieldset\Domain $domain;

    private \BayCMS\Fieldset\BayCMSList $list;

    private array $values;


    public function __construct(\BayCMS\Base\BayCMSContext $context, array $settings)
    {
        if (!isset($settings['class'])) {
            throw new \BayCMS\Exception\missingData('You have to give a class');
        }

        $this->context = $context;
        $this->settings = $settings;

    }

    public function createList()
    {
        $this->list = new \BayCMS\Fieldset\BayCMSList($this->context, [
            'from' => 'qb2_document t, objekt_verwaltung' . $this->context->get('row1', 'id') . ' o, qb2 q',
            'where' => 't.id=o.id and t.id_query=q.id and q.class=\'' .
                pg_escape_string($this->context->getDbConn(), $this->get('class')) . '\'',
            'id_query' => 't.id',
            'write_access_query' => 'check_objekt(t.id,' . $this->context->get('row1', 'id_benutzer', 0) . ')',
            'actions' => ['edit'],
            'view' => true
        ]);
        $this->list->addField(new TextInput($this->context, [
            'name' => 'Name'
        ]));
        $this->list->addField(new TextInput($this->context, [
            'name' => 'Mode'
        ]));
        $this->list->addField(new TextInput($this->context, [
            'name' => 'Query',
            'sql' => 'q.name'
        ]));
    }

    public function createQB()
    {
        if (!isset($this->settings['id_query']) && isset($_REQUEST['id_query'])) {
            $this->settings['id_query'] = $_REQUEST['id_query'];
        }

        if (!isset($this->settings['id_query']) && isset($_GET['id'])) {
            $res = pg_query_params(
                $this->context->getDbConn(),
                'select id_query from qb2_document where id=$1',
                [$_GET['id']]
            );
            if (pg_num_rows($res))
                [$this->settings['id_query']] = pg_fetch_row($res, 0);
        }

        if (!isset($this->settings['id_query'])) {
            throw new \BayCMS\Exception\missingData('You have to give a query id');
        }
        $res = pg_query_params(
            $this->context->getDbConn(),
            'select class from qb2 where id=$1',
            [$this->settings['id_query']]
        );
        if (!pg_num_rows($res))
            throw new \BayCMS\Exception\notFound('Query with ID ' . $this->settings['id_query'] . ' does not exist');
        [$class] = pg_fetch_row($res, 0);
        $this->qb = new $class($this->context);
        $this->qb->load($this->settings['id_query']);

    }

    public function createDomain()
    {
        if (!isset($this->qb))
            $this->createQB();
        $this->domain = new \BayCMS\Fieldset\Domain($this->context, [
            'table' => 'qb2_document',
            'uname' => 'qb2_document',
            'export' => false
        ]);

        $this->domain->addField(new \BayCMS\Field\Hidden($this->context, [
            'name' => 'id_query',
            'default_value' => $this->settings['id_query']
        ]));

        if (isset($_POST['id_query']))
            $_POST['id_parent'] = $_POST['id_query'];

        $this->domain->addField(new \BayCMS\Field\TextInput($this->context, [
            'name' => 'Name',
            'non_empty' => 1,
            'lfield' => 1,
            'sfield' => 1
        ]));
        $this->domain->addField(new \BayCMS\Field\Select($this->context, [
            'name' => 'Mode',
            'values' => [
                ['inc', $this->t('incremental', 'incrementell')],
                ['full', $this->t('always full', 'immer alle')]
            ],
            'non_empty' => 1,
            'lfield' => 1
        ]));
        $this->domain->addField(new \BayCMS\Field\Select($this->context, [
            'name' => 'min_power',
            'description' => $this->t('Download allowed for', 'Herunterladbar durch'),
            'db_query' => 'select power as value,
                 non_empty(' . $this->context->get('lang') . ',' . $this->context->get('lang2') . ') as description
                 from power where power<=1000 order by 1 desc',
            'default_value' => 1000,
            'non_empty' => 1
        ]));
        $this->domain->addField(new \BayCMS\Field\Number($this->context, [
            'name' => 'count',
            'description' => $this->t('Number of datasets per Page', 'Anzahl der Datensätze pro Seite'),
            'non_empty' => 1,
            'default_value' => 1
        ]));
        $this->domain->addField(new \BayCMS\Field\Comment($this->context, [
            'name' => 'replacement',
            'description' => $this->qb->getReplacementTable(),
            'not_in_table' => true
        ]));

        $this->domain->addField(new \BayCMS\Field\UploadFile($this->context, [
            'name' => 'template',
            'description' => $this->t('Template File', 'Template Datei') . ' (pdf, docx, pptx, odt, odp)',
            'non_empty' => 1,
            'preg_match' => '/(pdf|docx|odt|odp|pptx)/i'
        ]));

        $this->domain->addField(new \BayCMS\Field\Comment($this->context, [
            'name' => 'runs',
            'description' => "<h4>Runs</h4>",
            'not_in_form' => true
        ]));



    }

    public function getValueArray($id)
    {
        $this->load($id);
        $this->createQB();
        $exclude = ($this->values['mode'] == 'inc' ? $id : false);
        $list = $this->qb->getList($exclude);
        $qb_values = $list->getValues();

        $nr = 1;
        $values = [];
        $i = 0;
        $pages = ceil(count($qb_values) / $this->values['count']);
        foreach ($qb_values as $row) {
            $values[$i]['__ids'][] = $row['__id'];
            $values[$i]['__page_number'] = $i + 1;
            $values[$i]['__total_pages'] = $pages;
            foreach ($row as $key => $v) {
                $values[$i]['f' . $nr . '_' . $key] = $v;
            }
            $nr++;
            if ($nr > $this->values['count']) {
                $i++;
                $nr = 1;
            }
        }
        return $values;
    }

    public function load($id)
    {
        $res = pg_query_params(
            $this->context->getDbConn(),
            'select d.*,f.name as template
            from qb2_document d, objekt o, file f, objekt of
        where d.id=o.id and o.geloescht is null 
        and of.id_obj=o.id and f.id=of.id and f.de=\'template\'
        and d.id=$1',
            [$id]
        );
        if (!pg_num_rows($res))
            throw new \BayCMS\Exception\notFound('There is no document with id=' . $id);
        $this->values = pg_fetch_array($res, 0);
        if ($this->context->get('row1', 'power') < $this->values['min_power'])
            throw new \BayCMS\Exception\accessDenied('Your rights are not sufficient to read the document');
    }

    public function page($settings = [])
    {


        if ($_GET['id_file'] ?? false) {
            $file = new \BayCMS\Base\BayCMSFile($this->context);
            $file->load($_GET['id_file']);
            $f = $file->get();
            if (($_GET['aktion'] ?? false) == 'del'){
                $file->erase();
                $obj= new \BayCMS\Base\BayCMSObject($this->context);
                $obj->load($f['id_parent']);
                $obj->erase();
                unset($_GET['aktion']);
            }
            else {
                $p = new \BayCMS\Page\BinPage($this->context, [
                    'file' => $this->context->get('BayCMSRoot') . '/' . $f['full_path'] . '/' . $f['name'],
                    'file_name' => $f['name']
                ]);
                $p->page();
            }
        }

        $action = $_GET['aktion'] ?? false;
        if ($action == 'preview' || $action == 'json') {
            $val = $this->getValueArray($_GET['id']);
            $writer = new \BayCMS\Util\DocWriter(
                $this->context,
                $val,
                $this->context->get('BayCMSRoot') . '/' . $this->values['template']
            );
            if ($action == 'preview') {
                $res = $writer->write();
                $writer->send(
                    $res['tmp_name'],
                    $res['name'],
                    true
                );
                //Script exits here
            }
            header('Content-Type: text/event-stream');
            header('Cache-Control: no-cache');
            $res = $writer->write(true); //sends JSON

            $new_name = tempnam($this->context->get('BayCMSRoot') . '/tmp', 'qb2');
            rename($res['tmp_name'], $new_name);
            $res['tmp_name'] = $new_name;
            $obj = new \BayCMS\Base\BayCMSObject($this->context);
            $obj->set([
                'de'=>$this->values['name'],
                'id_parent' => $_GET['id'],
                'uname'=>'qb2_run'
            ]);
            $id_run =$obj->save();
            $file = new \BayCMS\Base\BayCMSFile($this->context);
            $file->set([
                'de' => $this->values['name'],
                'id_parent' => $id_run,
                'source' => $res['tmp_name'],
                'name' => $res['name'],
                'path' => 'inc/gru/qb2',
                'add_id_obj' => 1
            ]);
            $file->save();
            pg_query_params(
                $this->context->getRwDbConn(),
                'insert into qb2_run(id,name) values ($1,$2)',
                [$id_run, $this->values['name']]
            );

            pg_query($this->context->getRwDbConn(), 'begin');
            pg_prepare(
                $this->context->getRwDbConn(),
                'insert_qb2_rows',
                'insert into qb2_rows(id_run,id_obj,id_row) values($1,$2,$3)'
            );
            foreach ($val as $v) {
                foreach ($v['__ids'] as $id) {
                    $id_obj = (isset($this->settings['uname']) ? $id : null);
                    pg_execute(
                        $this->context->getRwDbConn(),
                        'insert_qb2_rows',
                        [$id_run, $id_obj, $id]
                    );
                }

            }
            pg_query($this->context->getRwDbConn(), 'commit');
            if (is_readable($res['tmp_name']))
                unlink($res['tmp_name']);
            exit();
        }

        $this->context->printHeader();
        echo $settings['pre_content'] ?? '';
        if ($action) {
            $this->createDomain();
            $edit = $this->domain->getEdit();
            if ($edit['edit']) {
                echo $edit['html'];
                $this->context->printFooter();
            } else {
                $_GET['id'] = $edit['id'];
            }
        }

        if ($action == 'write') {
            $this->printJSEventSource();
            $this->context->printFooter();
        }

        if (isset($_GET['id'])) {
            $this->createQB();
            $list = $this->qb->getList($_GET['id']);
            if ($num = $list->getNumRows()) {
                echo "<h3>" . $num . ' affected rows</h3>';
                echo $GLOBALS['TE']->getActionLink(
                    '?id=' . $_GET['id'] . '&aktion=edit',
                    $this->t('Edit', 'Bearbeiten'),
                    '',
                    'edit'
                );
                echo ' ';
                echo $GLOBALS['TE']->getActionLink(
                    '?id=' . $_GET['id'] . '&aktion=preview',
                    $this->t('Preview', 'Vorschau'),
                    '',
                    'eye'
                );
                echo ' ';
                echo $GLOBALS['TE']->getActionLink(
                    '?id=' . $_GET['id'] . '&aktion=write',
                    $this->t('Write document', 'Dokument schreiben'),
                    ' onClick="return confirm(\'' . $this->t('Are you sure', 'Sind Sie sicher') . '?\')"',
                    'square-download'
                );
                echo '<br/>';

                echo $list->getTable();
            } else {
                $GLOBALS['TE']->printMessage('nothing to do');
            }



            //print Table with runs...
            $list = new \BayCMS\Fieldset\BayCMSList($this->context, [
                'from' => 'objekt of, file f, objekt o, qb2_document d, benutzer b',
                'where' => 'f.id=of.id and of.id_benutzer=b.id and of.id_obj=o.id and o.id_obj=d.id and d.id=' . $_GET['id'],
                'actions' => ['del', 'view'],
                'qs' => 'id=' . $_GET['id'],
                'id_name' => 'id_file',
                'id_query' => 'f.id',
                'order_by' => ['o.ctime desc'],
                'view' => 1
            ]);
            $list->addField(new TextInput($this->context, [
                'name' => 'Date',
                'sql' => "to_char(o.ctime,'YYYY-MM-DD HH24:MI')"
            ]));
            $list->addField(new TextInput($this->context, [
                'name' => 'User',
                'sql' => 'b.kommentar'
            ]));
            echo $list->getTable();



            $this->context->printFooter();
        }

        $this->createList();
        echo $this->list->getTable();
        echo $settings['post_content'] ?? '';
        $this->context->printFooter();
    }


    private function printJSEventSource()
    {
        echo ' <div id="contact-results">Starting...</div>
        <br />

        <progress id="contact-progressor" value="0" max="100" style="width:80%;"></progress>
        <span id="contact-percentage" style="text-align:right; display:block; margin-top:5px;">0</span>
        ';
        echo "
<script>
";
        require_once __DIR__ . '/../Util/JS/EventSource.js';
        echo "</script>
        
<script>
function printLog(message) {
var r = document.getElementById('contact-results');
r.innerHTML = message;
}

if(typeof(EventSource) !== 'undefined') {
printLog('processing started');
          var es = new EventSource('?id=" . $_GET['id'] . "&aktion=json');

    //a message is received
    es.addEventListener('message', function(e) {
    var result = JSON.parse( e.data );

    if(result.message>'')
    printLog(result.message);

    if(e.lastEventId == 'CLOSE') {
    printLog('Finished');
    es.close();
    var pBar = document.getElementById('contact-progressor');
    pBar.value = pBar.max; //max out the progress bar
    window.setTimeout(window.location.href = '?id=" . $_GET['id'] . "',5000);

    }
    else {
    var pBar = document.getElementById('contact-progressor');
    pBar.value = result.progress;
    var perc = document.getElementById('contact-percentage');
    perc.innerHTML   = result.progress  + '%';
    perc.style.width = (Math.floor(pBar.clientWidth * (result.progress/100)) + 15) + 'px';
    }
    });

    es.addEventListener('error', function(e) {
    //printLog('Error occurred');
    es.close();

    });
    
    } else {
printLog('<b>Sorry! No server-sent events support. Please use another browser (e.g. Firefox).</b>');
    }
    
    </script>
    ";
    }
}